package com.ekingwin.bas.cloud.document.service.impl;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.annotation.Primary;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;

import com.ekingwin.bas.cloud.document.UAttachementException;
import com.ekingwin.bas.cloud.document.dto.UploadFileObject;
import com.ekingwin.bas.cloud.document.dto.UploadFileDto;
import com.ekingwin.bas.cloud.document.service.IIAttachmentService;
import com.ekingwin.bas.cloud.document.service.IIDownloadService;
import com.ekingwin.bas.cloud.document.service.IIUploaderService;
import com.ekingwin.bas.cloud.document.utils.DownloadUtils;
import com.ekingwin.bas.cloud.document.utils.ZipFileUtils;

@Service
@Primary
public class AttachmentServiceImpl implements IIAttachmentService {
	
	
	private Logger logger = LoggerFactory.getLogger(this.getClass());
	
	@Autowired
	@Qualifier(value="commonDownloadService")
	private IIDownloadService downloadService;
	@Autowired
	@Qualifier(value="commonUploaderService")
	private IIUploaderService uploaderService;
	
	
	public UploadFileObject downloadFile(String id) throws UAttachementException {
		return downloadService.downloadFile(id);
	}

	public UploadFileDto upload(File file, String fileName,
			Map<String, String[]> fileMeta) throws UAttachementException {
		return uploaderService.upload(file, fileName, fileMeta);
	}

	@Override
	public UploadFileDto upload(InputStream inputStream, String name, Map<String, String[]> fileMeta) throws UAttachementException {
		return uploaderService.upload(inputStream, name, fileMeta);
	}

	public void deleteFile(String id) throws UAttachementException {
		if(StringUtils.isEmpty(id)){
			throw new UAttachementException("当前附件的ID为空");
		}
//		UcmUser user = UcmUserService.getUserInfo(UserUtils.getCurrentUser());
		// 因上传后，删除页面会卡住，改用下面方法
		//DocService.deleteNoReconver(user, id);
//		DocService.deleteDocModelBydID(user, id);
	}

	@Override
	public UploadFileObject downloadFileBatch(String[] ids) throws UAttachementException {
		try {
			String tempZipName = "batchDownload";
			tempZipName = ZipFileUtils.buildFileName(tempZipName);
			UploadFileObject fileObj = new UploadFileObject();
			
			List<UploadFileObject> fileObjs = new ArrayList<UploadFileObject>();
			
			for (int i = 0; i < ids.length; i++) {
				UploadFileObject fo = this.downloadFile(ids[i]);
				fileObjs.add(fo);
			}

			File zipFile = ZipFileUtils.addMultiFileToZip(fileObjs, tempZipName);
			
			FileInputStream batchFis = new FileInputStream(zipFile);
			fileObj.setName(tempZipName);
			fileObj.setSize(zipFile.length());
			fileObj.setStream(batchFis);
			fileObj.setType("zip");
			return fileObj;
		} catch (Exception e) {
			logger.error(e.getMessage());
			throw new UAttachementException(e);
		}
	}
	
	

	@Override
	public ResponseEntity<byte[]> downloadFileByte(String id) throws UAttachementException {
		return downloadService.downloadFileByte(id);
	}

	@Override
	public ResponseEntity<byte[]> downloadFileBatchByte(String[] ids) throws UAttachementException {
		UploadFileObject fo = this.downloadFileBatch(ids);
		try {
			return DownloadUtils.download(fo);
		} catch (Exception e) {
			logger.error(e.getMessage());
			throw new UAttachementException(e);
		}
	}

	@Override
	public ResponseEntity<byte[]> downloadFileByte(String id, boolean b)  throws UAttachementException{
		return downloadService.downloadFileByte(id,b);
	}
}
