package com.ekingwin.bas.cloud.login.service.impl;

import com.ekingwin.bas.cloud.handler.BusinessException;
import com.ekingwin.bas.cloud.login.config.OauthConfig;
import com.ekingwin.bas.cloud.login.dao.LoginUserDao;
import com.ekingwin.bas.cloud.login.dto.OauthLoginUserDto;
import com.ekingwin.bas.cloud.login.service.ISsoService;
import com.ekingwin.bas.cloud.login.vo.LoginUserVo;
import com.ekingwin.bas.cloud.util.Client;
import com.ekingwin.bas.cloud.util.StringUtils;
import org.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Primary;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * @ClassName :
 * @description:
 * @author: zhangjie
 * @create: 2019-10-17 13:35
 **/
@Service
@Primary
@Transactional(rollbackFor = Exception.class)
public class ISsoServiceImpl implements ISsoService {

    private Logger logger = LoggerFactory.getLogger(ISsoServiceImpl.class);


    @Autowired
    private OauthConfig oauthConfig;


    @Autowired
    private LoginUserDao loginUserDao;





    /** 
    *
    * @Param:  * @param oauthLoginUserDto 
    * @return: com.ekingwin.bas.cloud.login.vo.LoginUserVo
    * @Author: zhangjie 
    * @Date: 2019/10/17 16:09
    */
    @Override
    public LoginUserVo  oauthLoginUser(OauthLoginUserDto oauthLoginUserDto) throws Exception{
        String accessToken=oauthLoginUserDto.getAccessToken();
        String requestUrl=oauthConfig.getAccessTokenOauth()+"?access_token="+accessToken;
        Client client =new Client();
        JSONObject jsonObject=client.getCode(requestUrl);
        logger.info(accessToken+":token信息》》》》》》》》》"+jsonObject.toString());
        //判断是否存在异常，和id是否为空
        if (jsonObject==null || jsonObject.has("error") || !jsonObject.has("id") || jsonObject.isNull("id")){
            throw new BusinessException(500,"登录信息超时，请重新登录");
        }
        //
        String userId=jsonObject.getString("id");
  /*      LoginUserVo userDetail = loginUserDao.getUserDetail(userId);
        if (userDetail ==null){
            throw new BusinessException(500,"TOKEN失效，请重新登录");
        }
        userDetail.setUserid(userId);*/
        return this.getUserDetailById(userId);

    }


    /**
    * 获取用户信息
    * @Param:  * @param userId
    * @return: com.ekingwin.bas.cloud.login.vo.LoginUserVo
    * @Author: zhangjie
    * @Date: 2019/10/21 21:04
    */

    @Override
    public String getUserArea(String userId) throws Exception{
        String areaname="E0103";
        LoginUserVo userDetail = loginUserDao.getUserDetail(userId);
        String custType = userDetail.getCustType();
        if ("2".equals(custType)){
            String outUserArea = loginUserDao.getOutUserArea(userId);
            if (StringUtils.isEmpty(outUserArea)){
            }else {
                areaname= outUserArea;
            }
        }else{
            //判断是否是集团
            String userAreasByJt = loginUserDao.getUserAreasByJt(userId);
            if (StringUtils.isEmpty(userAreasByJt)){
                List<String> userOrg = loginUserDao.getUserOrg(userId);
                if (userOrg !=null && userOrg.size()>0){
                    Set<String> strs=new HashSet<>();
                    for (String s : userOrg) {
                        strs.add(s);
                    }
                    String joinStr = String.join(",", strs);
                    String userAreas = loginUserDao.getUserAreas(joinStr);
                    if (!StringUtils.isEmpty(userAreas)){
                        areaname=userAreas;
                    }
                }else{

                }
            }else {
                areaname=userAreasByJt;
            }
        }




        return  areaname;

    }

    @Override
    public LoginUserVo getUserDetailById(String userId) throws Exception{
        LoginUserVo userDetail = loginUserDao.getUserDetail(userId);
        if (userDetail ==null){
            throw new BusinessException(500,"登录信息超时，请重新登录");
        }
        userDetail.setUserid(userId);
        userDetail.setAreaName(this.getUserArea(userId));
        return  userDetail;
    }


    /**
    * 保存下载量
    * @Param:  * @param apptype
    * @return: int
    * @Author: zhangjie
    * @Date: 2019/11/13 10:58
    */
    @Override
    public int insertAppDown(String apptype){

        return  loginUserDao.insertAppDown(apptype);
    }


    /**
    * 记录登录信息
    * @Param:  * @param apptype
     * @param userid
    * @return: int
    * @Author: zhangjie
    * @Date: 2019/11/13 11:00
    */
    @Override
    public int insertLoginInfo(String apptype,String userid){

        return  loginUserDao.insertLoginInfo(userid,apptype);
    }
}
