package com.ekingwin.bas.cloud.login.web;

import com.ekingwin.bas.cloud.login.dto.OauthLoginUserDto;
import com.ekingwin.bas.cloud.login.service.ISsoService;
import com.ekingwin.bas.cloud.login.vo.LoginUserVo;
import com.ekingwin.bas.cloud.util.Result;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import net.sf.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

@RestController
@RequestMapping("/login")
@Api("cas登陆")
public class LoginController {

	@Autowired
	private ISsoService iSsoService;

	@RequestMapping(value = "/caslogin", method = RequestMethod.GET)
	public String caslogin() throws Exception {
		return "caslogin";
	}
	@RequestMapping(value = "/login", method = RequestMethod.GET)
	public String login() throws Exception {
		return "login";
	}


	@PostMapping(value = "/getOauthLoginUser")
	@ApiOperation(value = "根据token获取登录用户信息", notes = "根据token获取登录用户信息")
	public Result getOauthLoginUser(@RequestBody OauthLoginUserDto oauthLoginUserDto)
			throws Exception{
		LoginUserVo loginUserVo = iSsoService.oauthLoginUser(oauthLoginUserDto);
		return  Result.ok(loginUserVo);
	}

	@PostMapping(value = "/getLoginUser/{userId}")
	@ApiOperation(value = "根据用户ID获取登录用户信息", notes = "根据用户ID获取登录用户信息")
	public Result getOauthLoginUser(@PathVariable(value = "userId") String userId)
			throws Exception{
		LoginUserVo loginUserVo = iSsoService.getUserDetailById(userId);
		return  Result.ok(loginUserVo);
	}

	@PostMapping(value = "/getUserArea/{userId}")
	@ApiOperation(value = "根据用户ID获取所在区域集团", notes = "根据用户ID获取所在区域集团")
	public Result getUserArea(@PathVariable(value = "userId") String userId)
			throws Exception{
		String userArea = iSsoService.getUserArea(userId);
		JSONObject jsonObject =new JSONObject();
		jsonObject.put("userArea",userArea);
		return  Result.ok(jsonObject);
	}


	@PostMapping(value = "/insertAppDown/{apptype}")
	@ApiOperation(value = "app 下载量", notes = "app 下载量")
	public Result insertAppDown(@PathVariable(value = "apptype") String apptype)
			throws Exception{
		iSsoService.insertAppDown(apptype);
		return  Result.ok();
	}

	@PostMapping(value = "/insertLoginInfo/{apptype}/{userid}")
	@ApiOperation(value = "app 下载量", notes = "app 下载量")
	public Result insertLoginInfo(@PathVariable(value = "apptype") String apptype,
								  @PathVariable(value = "userid") String userid)
			throws Exception{
		iSsoService.insertLoginInfo(apptype,userid);
		return  Result.ok();
	}

}
