package com.ekingwin.bas.cloud.org.dao;

import com.ekingwin.bas.cloud.org.OrganizationException;
import com.ekingwin.bas.cloud.org.dao.entity.Organization;
import com.ekingwin.bas.cloud.org.dao.entity.OrganizationTree;
import com.ekingwin.bas.cloud.org.dao.entity.OrganizationUser;
import com.ekingwin.bas.cloud.org.vo.OrganizationCompanyVo;
import com.ekingwin.bas.cloud.user.dto.UserDto;
import com.ekingwin.bas.mapper.BasMapper;
import org.apache.ibatis.annotations.Param;
import tk.mybatis.mapper.entity.Example;

import java.util.List;
import java.util.Map;

public interface IOrganizationUserDao extends BasMapper<OrganizationUser> {
	
	public String getOrgUserId(@Param("userid")String userid, @Param("orgid")String orgid) throws OrganizationException ;
	
	public OrganizationUser getOrgUser(@Param("userid")String userid, @Param("orgid")String orgid) throws OrganizationException ;
	
	/**
	 * 批量更新
	 * @param updateList
	 * @throws OrganizationException
	 */
	public void batchUpdate(List<OrganizationUser> updateList);
	/**
	 * 批量新增
	 * @param insertList
	 */
	public void batchInsert(List<OrganizationUser> insertList) ;
	/**
	 * 查询组织的类型
	 * @param id
	 * @return
	 * @throws OrganizationException
	 */
	public List<Organization> getTypeByOrgid(@Param("id")String id) throws OrganizationException ;

	/**
	 * 查询用户和组织对应关系
	 * @param userid
	 * @param orgid
	 * @return
	 * @throws OrganizationException
	 */
	public List<Map<String, Object>> getOrgByUserid(@Param("userid")String userid,@Param("orgid")String orgid) throws OrganizationException ;

	/**
	 * 删除人员和组织的对应关系
	 * @param userid
	 * @param orgid
	 */
	public void deleteUserOrg(@Param("userid")String userid,@Param("orgid")String orgid) ;
	
    /**
	 * 查询租户下所有内部用户（不包含禁用状态的）
	 * @param example
	 * @param empuid
	 * @return
	 */
	public List<UserDto> selectUsersByEmpuid(@Param("example")Example example,@Param("empuid")String empuid,@Param("orgid")String orgid);

	/**
	 * 根据orgid查询组织信息
	 * @param orgid
	 * @return
	 */
	public OrganizationTree queryOrg(@Param("orgid")String orgid);



	/**
	 * 删除组织下所有人员
	 *
	 * @param orgid
	 */
	public void deleteUserByOrgId(@Param("orgid")String orgid);




	/**
	 * 删除组织下的人员
	 */
	public void deleteUser(@Param("userid")String userid,@Param("orgid")String orgid);

	/**
	 * 获取当前登录人工号
	 * @param userid
	 * @return
	 */
	List<OrganizationCompanyVo> getCompanyByUserId(@Param("userid")String userid);
}

