package com.ekingwin.bas.cloud.org.service.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.annotation.Primary;
import org.springframework.stereotype.Service;

import com.ekingwin.bas.cloud.infra.dto.FilterDTO;
import com.ekingwin.bas.cloud.infra.dto.QueryDTO;
import com.ekingwin.bas.cloud.infra.exception.BasException;
import com.ekingwin.bas.cloud.infra.service.BaseService;
import com.ekingwin.bas.cloud.infra.utils.db.PageCriteria;
import com.ekingwin.bas.cloud.infra.utils.web.UserUtils;
import com.ekingwin.bas.cloud.org.OrganizationException;
import com.ekingwin.bas.cloud.org.dao.IMainInOrgViewDao;
import com.ekingwin.bas.cloud.org.dao.IOrgAuthorityViewDao;
import com.ekingwin.bas.cloud.org.dao.IOrgForMainViewDao;
import com.ekingwin.bas.cloud.org.dao.IOrganizationAuthorityDao;
import com.ekingwin.bas.cloud.org.dao.entity.MainInOrgView;
import com.ekingwin.bas.cloud.org.dao.entity.OrgForMainView;
import com.ekingwin.bas.cloud.org.dao.entity.OrganizationAuthority;
import com.ekingwin.bas.cloud.org.dao.entity.ViewOrgAuthority;
import com.ekingwin.bas.cloud.org.dto.OrganizationAuthorityDto;
import com.ekingwin.bas.cloud.org.dto.OrganizationDto;
import com.ekingwin.bas.cloud.org.service.IOrganizationAuthorityService;
import com.ekingwin.bas.cloud.org.service.IOrganizationService;
import com.ekingwin.bas.cloud.user.dto.UserDto;
import com.ekingwin.bas.cloud.user.service.IUserMetaService;

import tk.mybatis.mapper.entity.Example;

@Service
@Primary
public class OrganizationAuthorityServiceImpl extends BaseService implements IOrganizationAuthorityService {
	@Autowired
	private IOrganizationAuthorityDao orgAutDao;
	@Autowired
	private IOrgAuthorityViewDao orgAutViewDao;
	@Autowired
	private IMainInOrgViewDao mainInOrgViewDao;
	
	@Autowired
	private IOrgForMainViewDao orgForMainViewDao;
	
	@Autowired
	@Qualifier(value = "userMetaServiceImpl")
	private IUserMetaService userService;
	
	@Autowired
	@Qualifier(value="organizationServiceImpl")
	private IOrganizationService orgService;
	
	
	@Override
	public boolean save(OrganizationAuthorityDto orgAutDto) throws OrganizationException {
		try {
			String id = orgAutDto.getId();
			String item = "改";
			save(orgAutDto, OrganizationAuthority.class, orgAutDao);
			if (StringUtils.isEmpty(id)) {
				item = "增";
			}
			} catch (Exception e) {
			log.error(e.getMessage(), e);
			throw new OrganizationException(e);
		}
		return true;
	}

	@Override
	public boolean delete(String id) throws OrganizationException {
		OrganizationAuthorityDto orgAutDto = get(id);
		orgAutDao.deleteByPrimaryKey(id);
		try {
			} catch (Exception e) {
			e.printStackTrace();
		}
		return true;
	}

	@Override
	public PageCriteria<OrganizationAuthorityDto> query(QueryDTO query)
			throws OrganizationException {
		try {
			boolean superAdmin = true;
			if(superAdmin){
				Example example = buildQuery(query, OrganizationAuthority.class);
				
				return query(query, () -> orgAutDao.selectByExample(example),
						OrganizationAuthorityDto.class);
			}else{
				String userId = UserUtils.getCurrentUser();
				FilterDTO filterDto = new FilterDTO();
				filterDto.setKey("puserid");
				filterDto.setValue(userId);
				query.addFilter(filterDto);
				Example example = buildQuery(query, ViewOrgAuthority.class);
				
				return query(query, () -> orgAutViewDao.selectByExample(example),
						OrganizationAuthorityDto.class);
			}
			
			
		} catch (BasException e) {
			log.error(e.getMessage(), e);
			throw new OrganizationException(e);
		}
	}

	@Override
	public OrganizationAuthorityDto get(String id) throws OrganizationException {
		try {
			OrganizationAuthorityDto orgAutDto = get(OrganizationAuthorityDto.class, id,
					orgAutDao);
			return orgAutDto;
		} catch (Exception e) {
			log.error(e.getMessage(), e);
			throw new OrganizationException(e.getMessage(), e);
		}
	}
    
	/**
	 * 保存具有多个授权组织与被授权人的组织权限
	 */
	@SuppressWarnings("null")
	@Override
	public boolean saveAll(OrganizationAuthorityDto orgAutDtos) throws OrganizationException{
		String userIds = orgAutDtos.getUserid();
		String orgIds = orgAutDtos.getOrgid();
		
		String[] userIdArry = userIds.split(",");
		String[] orgIdArry = orgIds.split(",");
		
		String createby = UserUtils.getCurrentUser();
		orgAutDtos.setCreateby(createby);
		
		if(userIdArry == null && userIdArry.length==0){
			throw new OrganizationException("被授权人id不能为空！");
		}
		
		if(orgIdArry == null && orgIdArry.length==0){
			throw new OrganizationException("授权组织id不能为空！");
		}
		
		for(int i=0;i<userIdArry.length;i++){
			for(int j=0;j<orgIdArry.length;j++){
				OrganizationAuthorityDto orgAutDto = new OrganizationAuthorityDto();
				if(!StringUtils.isEmpty(orgAutDtos.getId())){
					orgAutDto.setId(orgAutDtos.getId());
				}				
				orgAutDto.setUserid(userIdArry[i]);
				orgAutDto.setOrgid(orgIdArry[j]);
				orgAutDto.setEffectivetime(orgAutDtos.getEffectivetime());
				orgAutDto.setFailuretime(orgAutDtos.getFailuretime());
				orgAutDto.setEnableSublicense(orgAutDtos.getEnableSublicense());
				orgAutDto.setCreateby(orgAutDtos.getCreateby());
				try {
					orgAutDto.setOrgname(orgService.get(orgIdArry[j]).getData().getName());
					orgAutDto.setUsername(userService.get(userIdArry[i]).getData().getCnname());
					orgAutDto.setCreaterusername(userService.get(orgAutDtos.getCreateby()).getData().getCnname());
				} catch (BasException e1) {
					e1.printStackTrace();
				}
				
				try {
					save(orgAutDto);
					
				} catch (OrganizationException e) {
					
					e.printStackTrace();
				}
			}
		}
		return true;
		
	}
   
	//移除管理员
	@Override
	public boolean removeMaintainersFromOrg(Map<String, String> map) throws OrganizationException{
		orgAutDao.removeMaintainersFromOrg(map);
		return true;
	}

	@Override
	public OrganizationAuthorityDto getOrgAutByUserOrg(Map<String, String> map)
			throws OrganizationException {
		OrganizationAuthority orgAut = orgAutDao.getOrgAutByUserOrg(map);
		OrganizationAuthorityDto orgAutDto = new OrganizationAuthorityDto();
		try {
			super.commonBuild(orgAutDto, orgAut);
		} catch (Exception e) {
			
			e.printStackTrace();
		}
		return orgAutDto;
	}

	@Override
	public List<OrganizationAuthorityDto> getOrgAut()
			throws OrganizationException {
        List<OrganizationAuthority> list = orgAutDao.getOrgAut();
		
		List<OrganizationAuthorityDto> dtoList = new ArrayList<OrganizationAuthorityDto>();	
		for(OrganizationAuthority orgAut:list){
			OrganizationAuthorityDto dto = new OrganizationAuthorityDto();
			try {
				super.commonBuild(dto, orgAut);
				dtoList.add(dto);
			} catch (Exception e) {
				e.printStackTrace();
			}
			
		}
		return dtoList;
	}

	@Override
	public PageCriteria<OrganizationAuthorityDto> getOrgAutByUser(QueryDTO query,String userid,int flag)
			throws OrganizationException {
		try {
			Example example = buildQuery(query, OrganizationAuthority.class);
			if (flag>0) {
				return query(query, 
						() -> orgAutDao.getOrgAutAll(example), 
						OrganizationAuthorityDto.class);

			}else{
				return query(query, 
						() -> orgAutDao.getOrgAutByUser(example, userid), 
						OrganizationAuthorityDto.class);
			}
		} catch (Exception e) {
			log.error(e.getMessage(), e);
			throw new OrganizationException(e);
		}
	}

	//查询管理员的组织
	@Override
	public PageCriteria<OrganizationDto> queryOrgForMaintainer(QueryDTO query) throws OrganizationException {
		 
		try {
			Example example = buildQuery(query, OrgForMainView.class);
			return query(query,  
					() -> orgForMainViewDao.selectByExample(example), 
					OrganizationDto.class);
		} catch (Exception e) {
			log.error(e.getMessage(), e);
			throw new OrganizationException(e);
		}
	}
	
	//查询组织的管理员
	@Override
	public PageCriteria<UserDto> queryMaintainersInOrg(QueryDTO query) throws OrganizationException {
		try {
			Example example = buildQuery(query, MainInOrgView.class);
			return query(query,  
					() -> mainInOrgViewDao.selectByExample(example), 
					UserDto.class);
		} catch (Exception e) {
			log.error(e.getMessage(), e);
			throw new OrganizationException(e);
		}
	}

}
