package com.ekingwin.bas.cloud.org.service.impl;

import com.ekingwin.bas.cloud.infra.dto.BaseResult;
import com.ekingwin.bas.cloud.infra.dto.QueryDTO;
import com.ekingwin.bas.cloud.infra.exception.BasException;
import com.ekingwin.bas.cloud.infra.service.BaseService;
import com.ekingwin.bas.cloud.infra.utils.common.ResultUtils;
import com.ekingwin.bas.cloud.infra.utils.db.PageCriteria;
import com.ekingwin.bas.cloud.infra.utils.web.UserUtils;
import com.ekingwin.bas.cloud.org.OrganizationException;
import com.ekingwin.bas.cloud.org.dao.*;
import com.ekingwin.bas.cloud.org.dao.entity.*;
import com.ekingwin.bas.cloud.org.dto.*;
import com.ekingwin.bas.cloud.org.service.IOrganizationService;
import com.ekingwin.bas.cloud.org.service.IOrganizationTreeService;
import com.ekingwin.bas.cloud.user.UserException;
import com.ekingwin.bas.cloud.user.dao.UserDao;
import com.ekingwin.bas.cloud.user.dao.entity.User;
import com.ekingwin.bas.cloud.user.dto.UserDto;
import com.github.pagehelper.util.StringUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.annotation.Primary;
import org.springframework.stereotype.Service;
import tk.mybatis.mapper.entity.Example;

import java.util.*;

@Service
@Primary
public class OrganizationServiceImpl extends BaseService implements IOrganizationService {

	@Autowired
	private IOrganizationDao orgDao;
	@Autowired
	private IOrganizationTreeDao OrgTreeDao;
	@Autowired
	@Qualifier(value = "organizationTreeServiceImpl")
	private IOrganizationTreeService orgTreeSer;



	@Autowired
	private IPostEsbDao postEsbDao;

	@Autowired
	private IOrganizationEsbDao organizationEsbDao;

		@Autowired
	private UserDao userDao;
	@Autowired
	private IOrganizationUserDao orgUserDao;

	@Override
	public BaseResult save(OrganizationDto organizationDto) throws OrganizationException {
		String orgid = organizationDto.getId();
		String orgpid = organizationDto.getPid();
		if("".equals(orgpid)) {
			organizationDto.setPid("1");
		}
		String item = "增";
		String operatorStaff = UserUtils.getCurrentUser();
		try {
			if (StringUtils.isEmpty(organizationDto.getCode())) {
				int row = 1;
				while (row > 0) {// 如果前端未传组织编码，系统生成4位数的随机数，并保证唯一性
					Random random = new Random();
					int s = random.nextInt(10000) % (10000 - 1000 + 1) + 1000;
					String code = Integer.toString(s);
					row = orgDao.judgeOrgCodeByCode(code);
					if (row == 0) {
						organizationDto.setCode(code);
					}
				}
			}
			
			if (StringUtils.isEmpty(organizationDto.getFullname())) {
				organizationDto.setFullname(organizationDto.getName());
			}
			save(organizationDto, Organization.class, orgDao);
			if (!StringUtils.isEmpty(orgid)) {
				item = "改";
			}
				OrganizationTreeDto orgTreeDto = new OrganizationTreeDto();
			String treeId = organizationDto.getId();
			orgTreeDto.setId(treeId);
			orgTreeDto.setName(organizationDto.getName());
			orgTreeDto.setPid(organizationDto.getPid());
			orgTreeDto.setType(organizationDto.getType());
			//获取treecode
			//TODO
			
			String treecode = orgDao.getTreeCodeById(organizationDto.getId());
			if(null==treecode) {
				treecode=orgDao.getNextTreeCodeByPid(organizationDto.getPid());
			}
			if(null==treecode) {
				treecode=orgDao.getTreeCodeById(organizationDto.getPid())+"001";
			}
			if(treecode.contains(".")){
				treecode = treecode.replace(".","");
			}
			orgTreeDto.setTreecode(treecode);
			orgTreeSer.save(orgTreeDto);
				// TODO 这里还要添加一个组织维度修改
			OrgDimensionDto odDto = new OrgDimensionDto();
			odDto.setName(organizationDto.getName());
			odDto.setId(organizationDto.getId());
				// TODO IDM12接口数据组织
			
		} catch (Exception e) {
			log.error(e.getMessage(), e);
			throw new OrganizationException(e);
		}
		OrganizationDto newDto = new OrganizationDto();
		newDto.setId(organizationDto.getId());
		return ResultUtils.build(newDto);
	}

	@Override
	public BaseResult delete(String id) throws OrganizationException {
		orgDao.deleteByPrimaryKey(id);
		return ResultUtils.build();
	}

	public void pushPostToEsb(OrganizationDto organizationDto,String isdelete) {
		try {
			int num = organizationEsbDao.getSystemCodeAndPostBySyscode("10005");
			if (num == 0) {
				return ;
			}
			int row = postEsbDao.getNumBySysCodes("10005", organizationDto.getPcode(), organizationDto.getCode(),
					organizationDto.getFullname(), organizationDto.getIsdelete(), organizationDto.getName());
			if (row == 0) {
				// 推送岗位IDM013
				IdmPostEsbDto peDto = new IdmPostEsbDto();
				peDto.setSysid("10005");
				peDto.setCode(organizationDto.getCode());
				peDto.setFullname(organizationDto.getFullname());
				peDto.setIsdelete(isdelete);
				peDto.setLinecode(organizationDto.getLinecode());
				peDto.setName(organizationDto.getName());
				peDto.setPcode(organizationDto.getPcode());
				peDto.setPpostcode(organizationDto.getPpostcode());
				peDto.setUpdateon(new Date());
				peDto.setStatus("0");
				peDto.setPushnum(0);
				save(peDto, IdmPostEsb.class, postEsbDao);
				String item = "增";
				if (isdelete.equals("N")) {
					item = "删";
				}
			
			}
		} catch (Exception e) {
			// TODO: handle exception
		}
	}

	public void pushOrganization(OrganizationDto organizationDto, String isdelete) {
		try {
			int num = organizationEsbDao.getSystemCodeAndOrgBySyscode("10005");
			if (num == 0) {
				return;
			}
			int row = organizationEsbDao.getNumBySyscodes("10005", organizationDto.getPcode(),
					organizationDto.getCode(), organizationDto.getFullname(), organizationDto.getIsdelete(),
					organizationDto.getType(), organizationDto.getName());
			if (row == 0) {
				OrganizationEsbDto oeDto = new OrganizationEsbDto();// 组织推送
				oeDto.setCode(organizationDto.getCode());
				oeDto.setFullname(organizationDto.getFullname());
				oeDto.setFunctionaty(null);
				oeDto.setIsdelete(isdelete);
				oeDto.setLinecode(organizationDto.getLinecode());
				oeDto.setName(organizationDto.getName());
				oeDto.setPcode(organizationDto.getPcode());
				oeDto.setStatus("0");
				oeDto.setPushnum(0);
				oeDto.setSysid("10005");
				oeDto.setType(organizationDto.getType());
				oeDto.setUpdateon(new Date());
				save(oeDto, OrganizationEsb.class, organizationEsbDao);
				String item = "增";
				if (isdelete.equals("N")) {
					item = "删";
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	@Override
	public BaseResult<PageCriteria<OrganizationDto>> query(QueryDTO query) throws OrganizationException {
		try {
			Example example = buildQuery(query, Organization.class);
			PageCriteria<OrganizationDto> result = query(query, () -> orgDao.selectByExample(example), OrganizationDto.class);
			return ResultUtils.build(result);	
		} catch (BasException e) {
			log.error(e.getMessage(), e);
			throw new OrganizationException(e);
		}
	}

	@Override
	public BaseResult<OrganizationDto> get(String id) throws OrganizationException {
		try {
			OrganizationDto organizationDto = get(OrganizationDto.class, id, orgDao);
			return ResultUtils.build(organizationDto);
		} catch (Exception e) {
			log.error(e.getMessage(), e);
			throw new OrganizationException(e.getMessage(), e);
		}
	}

	@Override
	public OrganizationDto getUserDept(String userId) throws OrganizationException {
		try {
			Organization organ = orgDao.getUserDept(userId);
			if (organ == null) {
				return null;
			}
			OrganizationDto organDto = new OrganizationDto();
			super.commonBuild(organDto, organ);
			return organDto;
		} catch (Exception e) {
			log.error(e.getMessage(), e);
			throw new OrganizationException(e.getMessage(), e);
		}

	}

	@Override
	public List<OrganizationDto> getUserPost(String userId) throws OrganizationException {
		List<Organization> list = orgDao.getUserPost(userId);

		List<OrganizationDto> dtoList = new ArrayList<OrganizationDto>();
		for (Organization organ : list) {
			try {
				OrganizationDto dto = new OrganizationDto();
				super.commonBuild(dto, organ);
				dtoList.add(dto);
			} catch (Exception e) {
				log.error(e.getMessage(), e);
				throw new OrganizationException(e.getMessage(), e);
			}
		}
		return dtoList;
	}

	@Override
	public boolean deleteById(String id) throws OrganizationException {
//		String allsubId=orgDao.getSubIdByPid(id);//获取到包含本身的ID及子ID
        orgDao.deleteBySubId(id);
        OrgTreeDao.batchDeleteById(id);
        String orgid=id;
		//删除组织下的所有用户
		orgUserDao.deleteUserByOrgId(orgid);
		return true;
	}

	@Override
	public List<OrganizationTreeDto> selectFirstTree() {
		String userId = UserUtils.getCurrentUser();
		List<OrganizationTree> list = orgDao.selectFirstTree(userId, userId);
		List<OrganizationTreeDto> dtoList = new ArrayList<OrganizationTreeDto>();
		for (OrganizationTree orgTree : list) {
			OrganizationTreeDto dto = new OrganizationTreeDto();
			try {
				super.commonBuild(dto, orgTree);
				dtoList.add(dto);
			} catch (Exception e) {
				e.printStackTrace();
			}

		}
		for (int i = 0; i < dtoList.size(); i++) {
			OrganizationTreeDto organizationTreeDto = dtoList.get(i);
			String orgtype = organizationTreeDto.getType();
			if (orgtype != null && "0".equals(orgtype)) {
				List<OrganizationTreeDto> newchildList = new ArrayList<OrganizationTreeDto>();
				organizationTreeDto.setChildList(newchildList);
				getChild(organizationTreeDto.getId(), dtoList, newchildList);
			}

		}
		List<OrganizationTreeDto> ndtoList = new ArrayList<OrganizationTreeDto>();
		for (OrganizationTreeDto dto : dtoList) {
			if (dto.getPid().equals("1")) {
				ndtoList.add(dto);
			}
		}
		return ndtoList;
	}

	/**
	 * 获取子集
	 * 
	 * @param pid
	 * @param organizationTreeList
	 * @param childList
	 */
	private void getChild(String pid, List<OrganizationTreeDto> organizationTreeList,
			List<OrganizationTreeDto> childList) {
		for (int i = 0; i < organizationTreeList.size(); i++) {
			OrganizationTreeDto organizationTreeDto = organizationTreeList.get(i);
			if (pid.equals(organizationTreeDto.getPid())) {
				childList.add(organizationTreeDto);
				List<OrganizationTreeDto> newchildList = new ArrayList<OrganizationTreeDto>();
				organizationTreeDto.setChildList(newchildList);
				getChild(organizationTreeDto.getId(), organizationTreeList, newchildList);
			}
		}
	}

	@Override
	public List<OrganizationTreeDto> getPartentTree(String id) throws OrganizationException {
		List<OrganizationTree> list = orgDao.getPartentTree(id);

		List<OrganizationTreeDto> dtoList = new ArrayList<OrganizationTreeDto>();
		for (OrganizationTree orgTree : list) {
			OrganizationTreeDto dto = new OrganizationTreeDto();
			try {
				super.commonBuild(dto, orgTree);
				dtoList.add(dto);
			} catch (Exception e) {
				e.printStackTrace();
			}

		}
		return dtoList;
	}

	// 获取当前用户的顶级组织
	@Override
	public List<OrganizationTreeDto> getLastParentTree() throws OrganizationException {
		List<OrganizationTreeDto> fList = selectFirstTree();

		List<OrganizationTreeDto> lList = new ArrayList<OrganizationTreeDto>();
		List<String> iList = new ArrayList<String>();
		for (OrganizationTreeDto orgTreeDto : fList) {
			List<OrganizationTreeDto> pList = getPartentTree(orgTreeDto.getId());

			for (OrganizationTreeDto orgTreeDtos : pList) {
				if (orgTreeDtos.getPid().equals("1")) {
					boolean flag = false;
					for (String id : iList) {
						if (id.equals(orgTreeDtos.getId())) {
							flag = true;
							break;
						}

					}
					if (!flag) {
						lList.add(orgTreeDtos);
						iList.add(orgTreeDtos.getId());
					}

				}
			}
		}
		return lList;
	}

	// 获取有权限的下一级组织
	@Override
	public List<OrganizationTreeDto> getChildTree(String id) throws OrganizationException {
		List<OrganizationTreeDto> cList =new ArrayList<OrganizationTreeDto>();
			try {
			cList = getChildTreeById(id);
		} catch (BasException e) {
			e.printStackTrace();
		}

		return cList;
	}

	// 获取所有下一级组织
	@Override
	public List<OrganizationTreeDto> getChildTreeById(String id) throws OrganizationException {
		List<OrganizationTree> list = orgDao.getChildTreeById(id);

		List<OrganizationTreeDto> dtoList = new ArrayList<OrganizationTreeDto>();
		for (OrganizationTree orgTree : list) {
			OrganizationTreeDto dto = new OrganizationTreeDto();
			try {
				super.commonBuild(dto, orgTree);
				dtoList.add(dto);
			} catch (Exception e) {
				e.printStackTrace();
			}

		}
		return dtoList;
	}

	// 查询所有下级组织
	@Override
	public List<OrganizationTreeDto> getAllChildTree(String id) throws OrganizationException {
		List<OrganizationTree> list = orgDao.getAllChildTree(id);

		List<OrganizationTreeDto> dtoList = new ArrayList<OrganizationTreeDto>();
		for (OrganizationTree orgTree : list) {
			OrganizationTreeDto dto = new OrganizationTreeDto();
			try {
				super.commonBuild(dto, orgTree);
				dtoList.add(dto);
			} catch (Exception e) {
				e.printStackTrace();
			}

		}
		return dtoList;
	}

	// 获取权限标志
	@Override
	public String getStringById(String id) throws OrganizationException {
		List<OrganizationTreeDto> fList = selectFirstTree();
		for (OrganizationTreeDto orgTreeDto : fList) {
			List<OrganizationTreeDto> acList = getAllChildTree(orgTreeDto.getId());
			for (OrganizationTreeDto acorgTreeDto : acList) {
				if (id.equals(acorgTreeDto.getId())) {
					return "1";
				}
			}
		}
		return "0";
	}

	@Override
	public List<PostDto> getPostById(String userid) throws OrganizationException {
		List<Map<String, Object>> mapList = orgDao.getPostById(userid);
		List<PostDto> list = new ArrayList<PostDto>();
		for (Map<String, Object> map : mapList) {
			PostDto dto = new PostDto();
			for (Map.Entry<String, Object> entry : map.entrySet()) {

				if ("ID".equals(entry.getKey())) {
					dto.setId(String.valueOf(entry.getValue()));
				}
				if ("NAME".equals(entry.getKey())) {
					dto.setName((String) entry.getValue());
				}
				if ("ISMAINPOST".equals(entry.getKey())) {
					dto.setPosttype((String) entry.getValue());
				}

			}
			if (!StringUtil.isEmpty(dto.getId())) {
				list.add(dto);
			}
		}

		return list;
	}

	// 获取下一级岗位
	@Override
	public List<OrganizationTreeDto> getChildPost(Map<String, String> map) throws OrganizationException {
		String postid = map.get("postid");
		String userid = map.get("userid");
		List<OrganizationTreeDto> treeList = getChildTreeById(postid);
		List<PostDto> postList = getPostById(userid);
		List<OrganizationTreeDto> list = new ArrayList<OrganizationTreeDto>();

		for (OrganizationTreeDto orgTreeDto : treeList) {
			boolean flag = true;
			for (PostDto postDto : postList) {
				if (orgTreeDto.getId().equals(postDto.getId())) {
					flag = false;
				}
			}
			if (flag) {
				list.add(orgTreeDto);
			}
		}
		return list;
	}

	@Override
	public PageCriteria<PostDto> queryPostById(QueryDTO query) throws OrganizationException {
		try {
			String userid = query.getAttr().get("userid");
			Example example = buildQuery(query, ViewUserPost.class);
			return query(query, () -> orgDao.queryPostById(example, userid), PostDto.class);
		} catch (BasException e) {
			log.error(e.getMessage(), e);
			throw new OrganizationException(e);
		}
	}

	@Override
	public String getOrgIdsByUserId(String empuid) throws OrganizationException {
		return orgDao.getOrgIdsByUserId(empuid);
	}

	/**
	 * 查询组织维度下面的岗位组织
	 * 
	 * @throws OrganizationException
	 */
	@Override
	public List<OrganizationTreePostDto> getPostByOrgid(String logid, String orgid, int num)
			throws OrganizationException {
		List<OrganizationTreePostDto> ors = new ArrayList<>();
		try {
			if (num > 0) {
				ors = orgDao.getPostAll(orgid);
			} else {
				ors = orgDao.getPostByOrgid(logid, orgid);
			}
			return ors;
		} catch (Exception e) {
			e.printStackTrace();
			throw new OrganizationException();
		}
	}

	/**
	 * 通过点击的组织ID和登录用户ID,获取该组织下的所有员工
	 * 
	 * @throws UserException
	 */
	@Override
	public PageCriteria<UserDto> getUserByLogIdAndOrgid(QueryDTO query, String logid, String orgid)
			throws UserException {
		try {
			Example example = buildQuery(query, User.class);
			return query(query, () -> userDao.getUserByLogIdAndOrgid(example, logid, orgid), UserDto.class);
		} catch (Exception e) {
			throw new UserException();
		}
	}

	@Override
	public String getPostRouteByPostid(String postid) {
		return orgDao.getPostRouteByPostid(postid);
	}

	@Override
	public BaseResult<PageCriteria<UserDto>> getUsersByOrg(QueryDTO query) throws BasException {
		String orgid = query.getAttr().get("orgid");
		Example example = buildQuery(query, User.class);
		PageCriteria<UserDto> result = query(query, () -> orgDao.getUsersByOrg(example,orgid),
				UserDto.class);
		return ResultUtils.build(result);
	}

	@Override
	public BaseResult<OrganizationDto> syncOrg(OrganizationDto orgdto) throws OrganizationException {
      //根据组织编码统判断组织是否存在
	  //查询父组织编码，查看是否能查到
	  //	
		try {
			Organization neworg=new Organization();
			Organization org = orgDao.getOrganizaitonByCode(orgdto.getCode());
			Organization parentOrg = orgDao.getOrganizaitonByCode(orgdto.getPcode());//父组织
			if(null!=parentOrg)
			orgdto.setPcode(parentOrg.getCode());
			if(null !=org) {//如果组织已存在，则更新
				orgdto.setId(org.getId());
				super.commonBuild(neworg,orgdto);
				orgDao.updateByPrimaryKey(neworg);
			}else {
				super.commonBuild(neworg,orgdto);
				int id=orgDao.insertSelective(neworg)	;
				orgdto.setId(id+"");
			}
		} catch (Exception e) {
			log.error(e.getMessage(), e);
			e.printStackTrace();
			throw new OrganizationException(e);
		}
		
		return ResultUtils.build(orgdto);
		// TODO Auto-generated method stub
		
	}

	@Override
	public Organization getNewOrg(OrganizationDto orgdto) throws OrganizationException {
		Organization neworg=new Organization();
		try {
			/*Organization org = orgDao.getOrganizaitonByCode(orgdto.getCode());
			Organization parentOrg = orgDao.getOrganizaitonByCode(orgdto.getPcode());//父组织
			if(null!=parentOrg)
			orgdto.setPcode(parentOrg.getCode());
			if(null !=org) {//如果组织已存在，则更新
				orgdto.setId(org.getId());
				super.commonBuild(neworg,orgdto);*/
			String id = orgDao.getIdByCode(orgdto.getCode());
			String code = orgDao.getCodeByCode(orgdto.getPcode());//父组织
			if(null!=code)
			orgdto.setPcode(code);
			if(null !=id) {//如果组织已存在，则更新
				orgdto.setId(id);
				super.commonBuild(neworg,orgdto);
			}else {
				super.commonBuild(neworg,orgdto);
			}
		} catch (Exception e) {
			log.error(e.getMessage(), e);
			e.printStackTrace();
			throw new OrganizationException(e);
		}
		
		return neworg;
	}

	@Override
	public void batchUpdate(List<Organization> updateList) throws OrganizationException {
		// TODO Auto-generated method stub
		orgDao.batchUpdate(updateList);
	}

	@Override
	public void batchInsert(List<Organization> insertList) throws OrganizationException {
		// TODO Auto-generated method stub
		orgDao.batchInsert(insertList);
	}


}