package com.ekingwin.bas.cloud.org.web;


import java.util.Map;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;
import com.ekingwin.bas.cloud.attachment.dto.FileObject;
import com.ekingwin.bas.cloud.infra.dto.QueryDTO;
import com.ekingwin.bas.cloud.infra.dto.SaveResultDTO;
import com.ekingwin.bas.cloud.infra.utils.db.PageCriteria;
import com.ekingwin.bas.cloud.infra.web.BaseController;
import com.ekingwin.bas.cloud.org.OrganizationException;
import com.ekingwin.bas.cloud.org.dto.OrganizationDto;
import com.ekingwin.bas.cloud.org.service.IOrganizationManager;
import com.ekingwin.bas.cloud.user.dto.UserDto;

@RestController
@RequestMapping("/org/manager")
@Api("OrganizationManagerController相关api")
public class OrganizationManagerController extends BaseController{
	@Autowired
	@Qualifier(value="organizationManagerImpl")
	private IOrganizationManager orgManager;

	
	/** 
	 * 将用户添加到组织中
	 * 
	 * @param userIds
	 * @param orgId
	 * @throws OrganizationException 
	 */
	@RequestMapping(value="/adduser",method = RequestMethod.PUT)
	public SaveResultDTO assignUsersToOrg(@RequestBody Map<String, String> map) throws OrganizationException{	
	   
	   SaveResultDTO result = new SaveResultDTO();
		try {
			
			orgManager.assignUsersToOrg(map);
			result.setResult("1");
			result.setMessage(map.get("userIds"));
		} catch (OrganizationException e) {
			result.setResult("0");
			result.setMessage(e.getMessage());
		}
		return result;
	}
	
	/** 
	 * 添加岗位
	 * 
	 * @param userIds
	 * @param orgId
	 * @throws OrganizationException 
	 */
	@RequestMapping(value="/addPost",method = RequestMethod.PUT)
	public SaveResultDTO addPost(@RequestBody Map<String, String> map) throws OrganizationException{	
	   SaveResultDTO result = new SaveResultDTO();
		try {
			orgManager.assignOrgsToUser(map);
			result.setResult("1");
			result.setMessage(map.get("userIds"));
		} catch (OrganizationException e) {
			result.setResult("0");
			result.setMessage(e.getMessage());
		}
		return result;
	}

	
	/**
	 * 将用户移除组织
	 * @param map
	 * @throws OrganizationException
	 */
	@RequestMapping(value="/removeuser",method = RequestMethod.PUT)
	public SaveResultDTO removeUsersFromOrg(@RequestBody Map<String, String> map) throws OrganizationException{	
	   
	   SaveResultDTO result = new SaveResultDTO();
		try {
			orgManager.removeUsersFromOrg(map);
			result.setResult("1");
			result.setMessage(map.get("userIds"));
		} catch (OrganizationException e) {
			result.setResult("0");
			result.setMessage(e.getMessage());
		}
		return result;
	}
	
	/**
	 * 分配管理员
	 * @param map
	 * @throws OrganizationException
	 */
	@RequestMapping(value="/addmain",method = RequestMethod.POST)
	public void assignMaintainersToOrg(@RequestBody Map<String, String> map) throws OrganizationException{	
	   orgManager.assignMaintainersToOrg(map);
	}
	
	/**
	 * 移除管理员
	 * @param map
	 * @throws OrganizationException
	 */
	@RequestMapping(value="/removemain",method = RequestMethod.POST)
	public void removeMaintainersFromOrg(@RequestBody Map<String, String> map) throws OrganizationException{	
	   orgManager.removeMaintainersFromOrg(map);
	}
	

	/**
	 * 查询指定组织的用户
	 * @param query
	 * @return
	 * @throws Exception
	 */
	@RequestMapping(value = "/queryuser", method = RequestMethod.POST)
	public PageCriteria<UserDto> queryUsersInOrg(@RequestBody(required = false) QueryDTO query) throws Exception {
		return orgManager.queryUsersInOrg(query);
	}
	
	/**
	 * 查询不在指定组织的用户
	 * @param query
	 * @return
	 * @throws Exception
	 */
	@RequestMapping(value = "/querynotuser", method = RequestMethod.POST)
	public PageCriteria<UserDto> queryUsersNotInOrg(@RequestBody(required = false) QueryDTO query) throws Exception {
		return orgManager.queryUsersNotInOrg(query);
	}
	
	
	/**
	 * 查询指定组织的管理员
	 * @param query
	 * @return
	 * @throws Exception
	 */
	@RequestMapping(value = "/querymain", method = RequestMethod.POST)
	public PageCriteria<UserDto> queryMaintainersInOrg(@RequestBody(required = false) QueryDTO query) throws Exception {
		return orgManager.queryMaintainersInOrg(query);
	}
	
	/**
	 * 查询指定管理员所管理的组织
	 * @param query
	 * @return
	 * @throws Exception
	 */
	@RequestMapping(value = "/queryorg", method = RequestMethod.POST)
	public PageCriteria<OrganizationDto> queryOrgForMaintainer(@RequestBody(required = false) QueryDTO query) throws Exception {
		return orgManager.queryOrgForMaintainer(query);
	}
	
	/**
	 * 以excel表格导出组织
	 * @param query
	 * @return
	 * @throws Exception
	 */
	@RequestMapping(value = "/exportsorg", method = RequestMethod.POST)
	public FileObject exportsOrg(@RequestBody(required = false) QueryDTO query) throws Exception {
		return orgManager.exportsOrg(query);
	}
	
	/**
	 * 以excel表格导入组织
	 * @param query
	 * @return
	 * @throws Exception
	 */
	@RequestMapping(value = "/importsorg", method = RequestMethod.POST)
	public FileObject importsorg() throws Exception {
		return orgManager.importsOrg();
	}
	
	
	/**
	 * 以excel表格导出用户
	 * @param query
	 * @return
	 * @throws Exception
	 */
	@RequestMapping(value = "/exportsuser", method = RequestMethod.POST)
	public FileObject exportsOrgUser(@RequestBody(required = false) QueryDTO query) throws Exception {
		return orgManager.exportsOrgUser(query);
	}
	
	/**
	 * 以excel表格导入用户
	 * @param query
	 * @return
	 * @throws Exception
	 */
	@RequestMapping(value = "/importsuser", method = RequestMethod.POST)
	public FileObject importsUserToOrg() throws Exception {
		return orgManager.importsUserToOrg();
	}


	
	

}
