package com.ekingwin.bas.cloud.org.web;

import java.util.Date;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import com.ekingwin.bas.cloud.infra.dto.BaseResult;
import com.ekingwin.bas.cloud.infra.dto.QueryDTO;
import com.ekingwin.bas.cloud.infra.utils.db.PageCriteria;
import com.ekingwin.bas.cloud.infra.utils.web.WebUtils;
import com.ekingwin.bas.cloud.org.dto.OrganizationDto;
import com.ekingwin.bas.cloud.org.service.IOrganizationTenantService;
import com.ekingwin.bas.cloud.user.dto.UserDto;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

@RestController
@RequestMapping("/tenant/organization")
@Api("租户组织结构相关api")
public class OrganizationTenantController extends OrganizationController  {
	
	@Autowired
	@Qualifier(value="organizationTenantServiceImpl")
	IOrganizationTenantService orgTenantService ;
	 
	/**
	 * 查询本公司下的所有组织 
	 * @param query
	 * @return
	 * @throws Exception
	 */
	@RequestMapping(value = "/queryid", method = RequestMethod.POST)
	public BaseResult<PageCriteria<OrganizationDto>> query(@RequestBody(required = false) QueryDTO query) throws Exception {
		return orgTenantService.query(query);
	}
	
	/**
	 * 查询组织下的用户
	 * @param query
	 * @return
	 * @throws Exception
	 */
	@RequestMapping(value = "/getUserByOrg", method = RequestMethod.POST)
	@ApiOperation("查询组织下的用户")
	public BaseResult<PageCriteria<UserDto>> getUsersByOrg(@RequestBody(required = false) QueryDTO query) throws Exception {
		return orgTenantService.getUserByOrg(query);
	}
	
	@RequestMapping(value="/importorganization", method = RequestMethod.POST)
	@ApiOperation("excel导入用户")
	public BaseResult<UserDto> importorganization(@RequestParam(value = "file") MultipartFile file,@RequestParam(value = "pid") String pid) throws Exception{
		BaseResult<UserDto> baseResult = new BaseResult<UserDto>();
		baseResult.setTimestamp(String.valueOf(new Date().getTime()));
		baseResult.setPath(WebUtils.getRequest().getRequestURI());
		try {
			orgTenantService.importorganization(file,pid);
			baseResult.setStatus("200");
			baseResult.setData(null);
		} catch (Exception e) {
			baseResult.setStatus("500");
			baseResult.setMessage(e.getMessage());
		}
		return baseResult;
	}
	

}
