package com.ekingwin.bas.cloud.org.web;

import java.util.Date;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.ekingwin.bas.cloud.infra.dto.BaseResult;
import com.ekingwin.bas.cloud.infra.dto.QueryDTO;
import com.ekingwin.bas.cloud.infra.utils.db.PageCriteria;
import com.ekingwin.bas.cloud.infra.utils.web.WebUtils;
import com.ekingwin.bas.cloud.org.dto.OrganizationUserDto;
import com.ekingwin.bas.cloud.org.service.IOrganizationUserTenantService;
import com.ekingwin.bas.cloud.user.dto.UserDto;
import com.ekingwin.bas.cloud.user.service.IUserMetaService;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

@RestController
@RequestMapping("/org/tenantuser")
@Api("用户组织和用户的关系")
public class OrganizationUserTenantController extends OrganizationUserController {
	
	@Autowired
	@Qualifier(value = "organizationUserTenantServiceImpl")
	private IOrganizationUserTenantService orgUserTenantSer;

	@RequestMapping(value = "/saveUser", method = RequestMethod.POST)
	@ApiOperation("对组织用户增加与修改")
	public BaseResult<OrganizationUserDto> saveUser(@RequestBody OrganizationUserDto orguser) throws Exception {
		BaseResult<OrganizationUserDto> baseResult = new BaseResult<OrganizationUserDto>();
		baseResult.setTimestamp(String.valueOf(new Date().getTime()));
		baseResult.setPath(WebUtils.getRequest().getRequestURI());
		try {
			orgUserTenantSer.save(orguser);
			baseResult.setStatus("200");
			baseResult.setData(orguser);
		} catch (Exception e) {
			baseResult.setStatus("500");
			baseResult.setMessage(e.getMessage());
		}
		return baseResult;
	}
	
	@RequestMapping(value = "/deltete", method = RequestMethod.POST)
	@ApiOperation("对组织与用户的对应关系删除")
	public BaseResult<OrganizationUserDto> deleteUser(@RequestBody OrganizationUserDto orguser) throws Exception {
		BaseResult<OrganizationUserDto> baseResult = new BaseResult<OrganizationUserDto>();
		baseResult.setTimestamp(String.valueOf(new Date().getTime()));
		baseResult.setPath(WebUtils.getRequest().getRequestURI());
		try {
			orgUserTenantSer.deleteUser(orguser);
			baseResult.setStatus("200");
			baseResult.setData(orguser);
		} catch (Exception e) {
			baseResult.setStatus("500");
			baseResult.setMessage(e.getMessage());
		}
		return baseResult;
		}
	
	@RequestMapping(value = "/getUsers", method = RequestMethod.POST)
	@ApiOperation("根据租户id获取所有内部用户")
	public BaseResult<PageCriteria<UserDto>> getTenantUser(@RequestBody(required = false) QueryDTO query) throws Exception {
		return orgUserTenantSer.getTenantUser(query);
	}
	
	
}
