package com.ekingwin.bas.cloud.supplier.web;

import com.ekingwin.bas.cloud.dto.BasePageDto;
import com.ekingwin.bas.cloud.infra.dto.BaseResult;
import com.ekingwin.bas.cloud.infra.utils.web.WebUtils;
import com.ekingwin.bas.cloud.supplier.dto.SupplierDto;
import com.ekingwin.bas.cloud.supplier.service.ISupplierService;
import com.ekingwin.bas.cloud.supplier.vo.AccountCenterInfo;
import com.ekingwin.bas.cloud.supplier.vo.SupplierBankVo;
import com.ekingwin.bas.cloud.supplier.vo.SupplierPageInfoVo;
import com.ekingwin.bas.cloud.user.dto.PasswordInfoDto;
import com.ekingwin.bas.cloud.util.Result;
import com.github.pagehelper.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;

@RestController
@RequestMapping("/supplier")
@Api("供应商")
public class SupplierController {
	
	@Autowired
	@Qualifier(value = "supplierServiceImpl")
	private ISupplierService supplierService;


	@RequestMapping(value = "/createUser", method = RequestMethod.POST)
	@ApiOperation("创建供应商登陆账号")
	public  BaseResult<SupplierDto>   createUser(@RequestBody SupplierDto supplierDto) throws Exception {
		BaseResult<SupplierDto> baseResult = new BaseResult<SupplierDto>();
		baseResult.setTimestamp(String.valueOf(new Date().getTime()));
		baseResult.setPath(WebUtils.getRequest().getRequestURI());
		try {
			supplierService.createUser(supplierDto);
			baseResult.setStatus("200");
			baseResult.setData(supplierDto);
		} catch (Exception e) {
			baseResult.setStatus("500");
			baseResult.setMessage(e.getMessage());
		}
		return baseResult;
		
	}

	@PostMapping(value = "/getPhoneAndMail")
	@ApiOperation(value="根据用户账户获取邮箱和手机号")
	public Result getPhoneAndMail(@RequestBody PasswordInfoDto passwordInfoDto)throws Exception{
		return supplierService.getPhoneAndMail(passwordInfoDto);
	}

	@PostMapping(value="/forgetPwd")
	@ApiOperation(value = "忘记密码",notes = "供应商根据手机号重置密码")
	public Result forgetPwd(@RequestBody PasswordInfoDto passwordInfoDto) throws Exception{
		supplierService.forgetUserPwd(passwordInfoDto);
		return  Result.ok();
	}

	@PostMapping(value="/sendSmsCode")
	@ApiOperation("重置密码发送验证码")
	public Result sendSmsCode(@RequestBody PasswordInfoDto passwordInfoDto) throws Exception{
		return supplierService.sendSmsCode(passwordInfoDto);
	}

	@PostMapping(value="/updatePwd")
	@ApiOperation("修改密码")
	public Result updatePwd(@RequestBody PasswordInfoDto passwordInfoDto) throws Exception{
		return supplierService.updateUserPwd(passwordInfoDto);
	}

	@PostMapping(value="/getSupplierList")
	@ApiOperation(value = "获取供应商列表",notes = "获取供应商列表")
	public Result getSupplierList(@RequestBody BasePageDto basePageDto) throws Exception{
		PageInfo<SupplierPageInfoVo> pageInfo = supplierService.selectPage(basePageDto);
		return Result.ok(pageInfo);
	}

	@GetMapping(value = "/getAccountCenterInfo/{supplierId}")
	@ApiOperation(value = "获取供应商账号中心信息",notes = "获取供应商账号中心信息",response = AccountCenterInfo.class)
	public Result getAccountCenterInfo(@PathVariable(value = "supplierId") String supplierId) throws Exception{
		AccountCenterInfo accountCenterInfo = supplierService.getAccountCenterInfo(supplierId);
		return  Result.ok(accountCenterInfo);
	}

	@GetMapping(value = "/getSuppliersBank/{supplierId}")
	@ApiOperation(value = "获取供应商银行信息",notes = "获取供应商银行信息",response = SupplierBankVo.class)
	public Result getSuppliersBank(@PathVariable(value = "supplierId") String supplierId) throws Exception{
		List<SupplierBankVo> suppliersBank = supplierService.getSuppliersBank(supplierId);
		return  Result.ok(suppliersBank);
	}

	@PostMapping(value = "/updateAccountCenterInfo")
	@ApiOperation(value = "供应商修改账户中心信息",notes = "供应商修改账户中心信息")
	public Result updateAccountCenterInfo(@RequestBody AccountCenterInfo accountCenterInfo) throws Exception{
		supplierService.updateAccountCenterInfo(accountCenterInfo);
		return  Result.ok();
	}

	@ApiOperation(value = "同步供应商", notes = "同步供应商")
	@PostMapping(value = "/sapSuppliers")
	public Result settlementUpload(@RequestParam(value = "pageSize") int pagesize,@RequestParam(value = "pageNum") int pageNum) throws Exception{
		supplierService.initSapSuppliers(pagesize,pageNum);
		return Result.ok();
	}



}
