package com.ekingwin.bas.cloud.thirdinterface.orgsync.service.impl;

import com.ekingwin.bas.cloud.infra.dto.BaseResult;
import com.ekingwin.bas.cloud.infra.dto.QueryDTO;
import com.ekingwin.bas.cloud.infra.exception.BasException;
import com.ekingwin.bas.cloud.infra.service.BaseService;
import com.ekingwin.bas.cloud.infra.utils.common.DateUtil;
import com.ekingwin.bas.cloud.infra.utils.db.PageCriteria;
import com.ekingwin.bas.cloud.org.OrganizationException;
import com.ekingwin.bas.cloud.org.dto.OrganizationDto;
import com.ekingwin.bas.cloud.org.service.IOrganizationService;
import com.ekingwin.bas.cloud.thirdinterface.orgsync.dto.*;
import com.ekingwin.bas.cloud.thirdinterface.orgsync.service.ISyncOrgService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Primary;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.http.converter.HttpMessageConverter;
import org.springframework.http.converter.json.MappingJackson2HttpMessageConverter;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Service
@Primary
public class SyncOrgServiceImpl extends BaseService implements ISyncOrgService {

	private Logger logger = LoggerFactory.getLogger(SyncOrgServiceImpl.class);

	@Value("${sync.orgurl}")
	private String syncorgurl;
	@Autowired
	private RestTemplate restTemplate;
	
	@Autowired
	@Qualifier(value="organizationServiceImpl")
	private IOrganizationService orgService;

	@Override
	public int getDeptCount() throws OrganizationException {
		List<HttpMessageConverter<?>> messageConverters = new ArrayList<HttpMessageConverter<?>>();
		MappingJackson2HttpMessageConverter converter = new MappingJackson2HttpMessageConverter();
		List<MediaType> mediaTypes = new ArrayList<>();
		mediaTypes.add(MediaType.TEXT_PLAIN);
		converter.setSupportedMediaTypes(mediaTypes);
		messageConverters.add(converter);
		restTemplate.setMessageConverters(messageConverters);
		OrgParamDto ord = new OrgParamDto();
		ord.setTID("INQUIRY");
		ord.setAccount("syc");
		ord.setPassword("syc");
		ord.setTransName("D");
		ord.setMode("F");
		ord.setTransDate(DateUtil.addDay(new Date(), -1));
		ord.setFromNumber(1);
		// ord.setToNumber(20);
		ResponseEntity<ResponseDept> responseEntity = restTemplate.postForEntity(syncorgurl, ord, ResponseDept.class);
		logger.info(responseEntity.toString());
		ResponseDept respdept = responseEntity.getBody();
		return respdept.getRESPONSEHEAD().getCount();
	}

	@Override
	public DeptList getDeptList(int fromNumber, int toNumber) throws OrganizationException {
		List<HttpMessageConverter<?>> messageConverters = new ArrayList<HttpMessageConverter<?>>();
		MappingJackson2HttpMessageConverter converter = new MappingJackson2HttpMessageConverter();
		List<MediaType> mediaTypes = new ArrayList<>();
		mediaTypes.add(MediaType.TEXT_PLAIN);
		converter.setSupportedMediaTypes(mediaTypes);
		messageConverters.add(converter);
		restTemplate.setMessageConverters(messageConverters);
		OrgParamDto ord = new OrgParamDto();
		ord.setTID("GETDEPT");
		ord.setAccount("syc");
		ord.setPassword("syc");
		ord.setTransName("D");
		ord.setMode("F");
		ord.setTransDate(DateUtil.addDay(new Date(), -1));
		ord.setFromNumber(fromNumber);
		ord.setToNumber(toNumber);
		ResponseEntity<ResponseDept> responseEntity = restTemplate.postForEntity(
				syncorgurl, ord,
				ResponseDept.class);
		logger.info(responseEntity.toString());
		ResponseDept respdept = responseEntity.getBody();
		return respdept.getDEPTLIST();
	}

	@Override
	public BaseResult<OrganizationDto> save(OrganizationDto object) throws BasException {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BaseResult<OrganizationDto> delete(String id) throws BasException {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BaseResult<PageCriteria<OrganizationDto>> query(QueryDTO query) throws BasException {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BaseResult<OrganizationDto> get(String id) throws BasException {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public int getPersonCount() throws OrganizationException {
		List<HttpMessageConverter<?>> messageConverters = new ArrayList<HttpMessageConverter<?>>();
		MappingJackson2HttpMessageConverter converter = new MappingJackson2HttpMessageConverter();
		List<MediaType> mediaTypes = new ArrayList<>();
		mediaTypes.add(MediaType.TEXT_PLAIN);
		converter.setSupportedMediaTypes(mediaTypes);
		messageConverters.add(converter);
		restTemplate.setMessageConverters(messageConverters);
		OrgParamDto ord = new OrgParamDto();
		ord.setTID("INQUIRY");
		ord.setAccount("syc");
		ord.setPassword("syc");
		ord.setTransName("P");
		ord.setMode("F");
		ord.setTransDate(new Date());
		ord.setFromNumber(1);
		// ord.setToNumber(20);
		ResponseEntity<ResponsePerson> responseEntity = restTemplate.postForEntity(syncorgurl, ord, ResponsePerson.class);
		logger.info(responseEntity.toString());
		ResponsePerson respperson = responseEntity.getBody();
		return respperson.getResponsehead().getCount();
	}

	@Override
	public PersonList getPersonList(int fromNumber, int toNumber) throws OrganizationException {
		List<HttpMessageConverter<?>> messageConverters = new ArrayList<HttpMessageConverter<?>>();
		MappingJackson2HttpMessageConverter converter = new MappingJackson2HttpMessageConverter();
		List<MediaType> mediaTypes = new ArrayList<>();
		mediaTypes.add(MediaType.TEXT_PLAIN);
		converter.setSupportedMediaTypes(mediaTypes);
		messageConverters.add(converter);
		restTemplate.setMessageConverters(messageConverters);
		OrgParamDto ord = new OrgParamDto();
		ord.setTID("GETPERSON");
		ord.setAccount("syc");
		ord.setPassword("syc");
		ord.setTransName("P");
		ord.setMode("F");
		ord.setTransDate(new Date());
		ord.setFromNumber(fromNumber);
		ord.setToNumber(toNumber);
		ResponseEntity<ResponsePerson> responseEntity = restTemplate.postForEntity(syncorgurl, ord, ResponsePerson.class);
		logger.info(responseEntity.toString());
		ResponsePerson respperson = responseEntity.getBody();
			return respperson.getPersonlist();
	}

}