package com.ekingwin.bas.cloud.user.service.impl;

import com.ekingwin.bas.cloud.infra.utils.web.UserUtils;
import com.ekingwin.bas.cloud.rolejcy.service.IRoleegoService;
import com.ekingwin.bas.cloud.user.dao.IpcOutUserMapper;
import com.ekingwin.bas.cloud.user.dao.UserDao;
import com.ekingwin.bas.cloud.user.dao.entity.IpcOutUser;
import com.ekingwin.bas.cloud.user.dao.entity.User;
import com.ekingwin.bas.cloud.user.dto.IpcOutUserDto;
import com.ekingwin.bas.cloud.user.dto.IpcOutUserPageDto;
import com.ekingwin.bas.cloud.user.dto.UserToBpmDto;
import com.ekingwin.bas.cloud.user.service.IpcOutUserService;
import com.ekingwin.bas.cloud.util.Md5Util;
import com.ekingwin.bas.cloud.util.StringUtils;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Primary;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

@Service
@Primary
@Transactional(rollbackFor = Exception.class)
public class IpcOutUserServiceImpl implements IpcOutUserService  {

    @Autowired
    private IpcOutUserMapper ipcOutUserMapper;

    @Autowired
    @Qualifier("userMetaServiceImpl")
    private UserMetaServiceImpl userMetaService;

    @Autowired
    private IRoleegoService iRoleegoService;

    @Autowired
    private UserDao userDao;

    @Value("${defaultpos}")
    private String defaultpos;
    @Value("${defaultposbas}")
    private String defaultposbas;
    @Value("${deptid}")
    private String deptid;
    @Value("${defaultRole}")
    private String defaultRole;
    @Value("${internalRole}")
    private String internalRole;

    /**
     * 保存修改用户信息
     *
     * @param ipcOutUserDto
     * @return id
     */
    @Override
    public Long saveOrUpdate(IpcOutUserDto ipcOutUserDto) throws Exception{
        Long id = ipcOutUserDto.getId();
        if (StringUtils.isEmpty(id)) {
            String password = ipcOutUserDto.getPassword();
            password = Md5Util.getMd5Password(password);
            ipcOutUserDto.setPassword(password);
            ipcOutUserDto.setNewUser(UserUtils.getCurrentUser());
            //Y是内部员工同步插入
            if ("Y".equals(ipcOutUserDto.getMark())){
                //内部员工电话号码存在使用电话号码作为usermsg
                if (!StringUtils.isEmpty(ipcOutUserDto.getCallPhone())){
                    IpcOutUser outUser = ipcOutUserMapper.selectCallPhone(ipcOutUserDto);
                    //电话号码不存在
                    if (outUser == null){
                        ipcOutUserDto.setUsermsg(ipcOutUserDto.getCallPhone());
                    }else {
                        ipcOutUserMapper.updateOutUser(ipcOutUserDto);
                        return outUser.getId();
                    }
                }else {
                    //为空使用RC- 规则生成
                    String usermsg = ipcOutUserMapper.selectUsermsg(ipcOutUserDto);
                    if (!StringUtils.isEmpty(usermsg)){
                        //后端生成，规则RC-000001 在前面补0  共6位
                        String string = String.format("%06d", Integer.valueOf(usermsg));
                        usermsg = "RC-"+string ;
                        ipcOutUserDto.setUsermsg(usermsg);
                    }
                }
                //同步角色67
                iRoleegoService.insertAuthRole(ipcOutUserDto.getUsermsg(),internalRole);
            }else {
                String usermsg = ipcOutUserMapper.selectUsermsg(ipcOutUserDto);
                if (!StringUtils.isEmpty(usermsg)){
                    //后端生成，规则RC-000001 在前面补0  共6位
                    String string = String.format("%06d", Integer.valueOf(usermsg));
                    usermsg = "RC-"+string ;
                    ipcOutUserDto.setUsermsg(usermsg);
                }
                //同步角色52
                iRoleegoService.insertAuthRole(ipcOutUserDto.getUsermsg(),defaultRole);
            }
            int result = ipcOutUserMapper.insertSelective(ipcOutUserDto);

            //外部用户新增成功,同步用户到bpm
            if(result>0){
                UserToBpmDto ubd = new UserToBpmDto();
                if (StringUtils.isEmpty(ipcOutUserDto.getCallPhone())){
                    ubd.setEmpid(ipcOutUserDto.getContactspinyin());
                    ubd.setEmpuid(ipcOutUserDto.getUsermsg());
                }else {
                    ubd.setEmpid(ipcOutUserDto.getContactspinyin());
                    ubd.setEmpuid(ipcOutUserDto.getCallPhone());
                }
                ubd.setCnname(ipcOutUserDto.getContacts());
                ubd.setEmail(ipcOutUserDto.getEmail());
                ubd.setPhone(ipcOutUserDto.getCallPhone());
                ubd.setIsDeleted("N");
                ubd.setDisplayname(ipcOutUserDto.getContacts());
                ubd.setDeptid(deptid);
                ubd.setDefaultpos(Integer.valueOf(defaultposbas));
                ubd.setMobilephone(ipcOutUserDto.getCallPhone());
                ubd.setRemark("2");
                User user = new User();
                BeanUtils.copyProperties(ubd,user);
                userDao.saveUserToEmployee(user);
                Boolean flag = userMetaService.saveUserToBpm(ubd);
            }
            id = ipcOutUserDto.getId();
        } else {
            ipcOutUserMapper.updateByPrimaryKeySelective(ipcOutUserDto);
        }
        return id;
    }

    /**
     * 删除外部信息
     *
     * @param ipcOutUserDto
     */
    @Override
    public void delete(IpcOutUserDto ipcOutUserDto) {
        Long id = ipcOutUserDto.getId();
        ipcOutUserMapper.deleteByPrimaryKey(id);
    }

    @Override
    public IpcOutUserDto select(IpcOutUserDto ipcOutUserDto) {
        Long id = ipcOutUserDto.getId();
        ipcOutUserDto = new IpcOutUserDto();
        if (StringUtils.isEmpty(id)) {
            ipcOutUserMapper.selectByPrimaryKey(id);
        }
        return ipcOutUserDto;
    }

    @Override
    public PageInfo selectPage(IpcOutUserPageDto ipcOutUserPageDto) {
        PageHelper.startPage(ipcOutUserPageDto.getPagenum(), ipcOutUserPageDto.getPagesize());
        List<IpcOutUserDto> ipcOutUserDtos = ipcOutUserMapper.selectPage(ipcOutUserPageDto);
        PageInfo<IpcOutUserDto> pageInfo = new PageInfo<>(ipcOutUserDtos);
        return pageInfo;
    }


//    批量禁用
    @Override
    public void batchDisable(List<IpcOutUserDto> ipcOutUserDto) {
      for (IpcOutUserDto ipcOutUserDto1 : ipcOutUserDto){
          ipcOutUserDto1.setState(0);
          ipcOutUserMapper.updateState(ipcOutUserDto1);
      }
    }

//    批量启用
    @Override
    public void batchEnabled(List<IpcOutUserDto> ipcOutUserDto) {
        for (IpcOutUserDto ipcOutUserDto1 : ipcOutUserDto){
            ipcOutUserDto1.setState(1);
            ipcOutUserMapper.updateState(ipcOutUserDto1);
        }
    }

    /**
     * 启用禁用操作
     * 1表示启用，0表示禁用
     * @param ipcOutUserDto
     */
    @Override
    public void operation(IpcOutUserDto ipcOutUserDto) {

        int selectState = ipcOutUserMapper.selectState(ipcOutUserDto);
        if (selectState == 1){
            selectState = 0;
        }else{
            selectState = 1;
        }
        ipcOutUserDto.setState(selectState);
        ipcOutUserMapper.updateState(ipcOutUserDto);
    }

    public static void main(String[] args) {
        //生成usermsg
        String usermsg = "RC-73857";
        usermsg = usermsg.substring(1,usermsg.length());
        int length = usermsg.length();
        int index = 0;
        char strs[] = usermsg.toCharArray();
        for (int i = 0; i < length; i++) {
            if ('0' != strs[i]) {
                index = i;// 找到非零字符串并跳出
                if('C'==strs[i]){
                    index = i+2;
                }
                break;
            }
        }
        int Id = Integer.valueOf(usermsg.substring(index,length))+1;
        usermsg = "RC-"+Id ;
    }

    /**
     * 同步签约供应商
     * @param ipcOutUserDto
     * @throws Exception
     */
    @Override
    public void initIpCoutUser(IpcOutUserDto ipcOutUserDto) throws Exception{
        IpcOutUser ipcOutUser = ipcOutUserMapper.selectByUsermsg(ipcOutUserDto.getUsermsg());
        if (ipcOutUser ==null){
            ipcOutUserDto.setPassword("7c6a180b36896a0a8c02787eeafb0e4c");
            ipcOutUserMapper.insertSelective(ipcOutUserDto);
            //同步角色52
            iRoleegoService.insertAuthRole(ipcOutUserDto.getUsermsg(),defaultRole);
        }


    }

}
