package com.ekingwin.bas.cloud.user.service.impl;

import java.io.IOException;
import java.io.InputStream;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Random;

import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.ss.usermodel.WorkbookFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.annotation.Primary;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import com.ekingwin.bas.cloud.infra.dto.BaseResult;
import com.ekingwin.bas.cloud.infra.dto.QueryDTO;
import com.ekingwin.bas.cloud.infra.exception.BasException;
import com.ekingwin.bas.cloud.infra.service.BaseService;
import com.ekingwin.bas.cloud.infra.utils.common.ResultUtils;
import com.ekingwin.bas.cloud.infra.utils.db.PageCriteria;
import com.ekingwin.bas.cloud.infra.utils.web.UserUtils;
import com.ekingwin.bas.cloud.ldap.LdapException;
import com.ekingwin.bas.cloud.ldap.dto.LdapPersonDto;
import com.ekingwin.bas.cloud.ldap.service.ILdapPersonService;
import com.ekingwin.bas.cloud.org.dto.OrganizationDto;
import com.ekingwin.bas.cloud.org.service.IOrganizationService;
import com.ekingwin.bas.cloud.tenant.dao.TenantDao;
import com.ekingwin.bas.cloud.tenant.dto.TenantEmployeeDto;
import com.ekingwin.bas.cloud.tenant.service.ITenantEmployeeService;
import com.ekingwin.bas.cloud.tenant.service.ITenantService;
import com.ekingwin.bas.cloud.user.UserException;
import com.ekingwin.bas.cloud.user.dao.UserDao;
import com.ekingwin.bas.cloud.user.dao.entity.User;
import com.ekingwin.bas.cloud.user.dto.UserDto;
import com.ekingwin.bas.cloud.user.service.IUserMetaService;
import com.ekingwin.bas.cloud.user.service.IUserMetaTenantService;
import com.ekingwin.bas.cloud.util.StringUtils;

import tk.mybatis.mapper.entity.Example;

@Primary
@Service
public class UserMetaTenantServiceImpl extends BaseService implements IUserMetaTenantService {

	@Autowired
	private UserDao userDao;

	@Autowired
	private TenantDao tenantDao;

	@Autowired
	@Qualifier(value = "userMetaServiceImpl")
	private IUserMetaService userService;

	@Autowired
	@Qualifier(value = "ldapPersonServiceImpl")
	private ILdapPersonService ldapPersonService;

	@Autowired
	@Qualifier(value = "tenantEmployeeServiceImpl")
	private ITenantEmployeeService tenantEmployeeService;

	@Autowired
	@Qualifier(value = "tenantServiceImpl")
	private ITenantService tenantService;

	
	@Autowired
	private IOrganizationService orgService;

	@Override
	public BaseResult<UserDto> save(UserDto userDto) throws BasException {
		try {
			BaseResult<UserDto> result = null;
			boolean newuser = false;
			// 获取当前用户信息
			 String currentUserId = UserUtils.getCurrentUser();
			 BaseResult<UserDto> BaseCurrentUserDto = userService.get(currentUserId);
			 if (BaseCurrentUserDto == null || BaseCurrentUserDto.getData() == null) {
			 throw new BasException("请先登陆！");
			 }
			 UserDto currentUserDto = BaseCurrentUserDto.getData();
			 String tenantid = currentUserDto.getTenantid();
			if (tenantid == null) {
				tenantid = "0000000000";
			}
			// 设置租户信息（租户ID）
			userDto.setTenantid(tenantid);

			String empuid = userDto.getEmpuid();
			String email = userDto.getEmail();
			String mobilephone = userDto.getMobilephone();
			List<Map<String, Object>> employeeList = null;
			// 判断手机号是否被占用
			if (mobilephone != null) {
				employeeList = userDao.getEmployeesByMobilephoneEmpuidEmail(mobilephone);
				for (int i = 0; i < employeeList.size(); i++) {

					Map<String, Object> employeeMap = employeeList.get(i);
					Object newempuid = employeeMap.get("EMPUID");
					if (newempuid == null) {
						newempuid = employeeMap.get("empuid");
					}
					Object is_deleted = employeeMap.get("IS_DELETED");
					if (is_deleted == null) {
						is_deleted = employeeMap.get("is_deleted");
					}
					if (is_deleted == null) {
						is_deleted = "N";
					}
					if (empuid == null) {
						if ("N".equals(is_deleted.toString())) {
							throw new BasException("此手机号已被占用");
						} else {
							userDao.clearPhoneByEmpuid(newempuid.toString());
						}
					} else {
						if (!empuid.equals(newempuid)) {
							if ("N".equals(is_deleted.toString())) {
								throw new BasException("此手机号已被占用");
							} else {
								userDao.clearPhoneByEmpuid(newempuid.toString());
							}
						}
					}
				}
			}
			// 判断邮箱是否被占用
			if (email != null) {
				employeeList = userDao.getEmployeesByMobilephoneEmpuidEmail(email);
				for (int i = 0; i < employeeList.size(); i++) {

					Map<String, Object> employeeMap = employeeList.get(i);
					Object newempuid = employeeMap.get("EMPUID");
					if (newempuid == null) {
						newempuid = employeeMap.get("empuid");
					}
					Object is_deleted = employeeMap.get("IS_DELETED");
					if (is_deleted == null) {
						is_deleted = employeeMap.get("is_deleted");
					}
					if (is_deleted == null) {
						is_deleted = "N";
					}
					if (empuid == null) {
						if ("N".equals(is_deleted.toString())) {
							throw new BasException("此邮箱已被占用");
						} else {
							userDao.clearPhoneByEmpuid(newempuid.toString());
						}
					} else {
						if (!empuid.equals(newempuid)) {
							if ("N".equals(is_deleted.toString())) {
								throw new BasException("此邮箱已被占用");
							} else {
								userDao.clearPhoneByEmpuid(newempuid.toString());
							}
						}
					}
				}
			}
			// 如果用户账号为空，生成账号
			if (empuid == null || "".equals(empuid)) {
				newuser = true;
				// 生成empuid
				// 先创建虚拟phone
				boolean createphone = false;
				if (mobilephone == null) {
					Date now = new Date();
					Long newtime = now.getTime();
					mobilephone = "" + newtime + tenantid;
					createphone = true;
					Random ra = new Random();
					for (int i = 0; i < 5; i++) {
						mobilephone += ra.nextInt(10);
					}
				}
				TenantEmployeeDto tenantEmployeeDto = new TenantEmployeeDto();
				tenantEmployeeDto.setTenantid(tenantid);
				tenantEmployeeDto.setMobilephone(mobilephone);
				tenantEmployeeService.save(tenantEmployeeDto);
				tenantEmployeeDto = tenantEmployeeService.getByMobilephone(mobilephone);
				empuid = tenantEmployeeDto.getId();
				userDto.setEmpuid(empuid);
				// 删除虚拟手机信息
				if (createphone) {
					tenantDao.deleteTenantEmployeeByMobilephone(mobilephone);
				}
			}

			// 默认创建采购用户
			String password = userDto.getPassword();
			userDto.setPassword("");
			userDto.setBusinesstype("01");
			result = userService.save(userDto);
			// 人员数据同步到IDAP
			Map<String, String> map = new HashMap<String, String>();
			map.put("gidNumber", "100");
			map.put("homeDirectory", "/home/admin");
			map.put("sn", userDto.getEmpuid());
			map.put("uid", userDto.getEmpuid());
			map.put("uidNumber", "200");
			map.put("mail", userDto.getEmail());
			map.put("mobile", userDto.getMobilephone());
			map.put("givenName", userDto.getCnname());
			map.put("displayName", userDto.getDisplayname());
			password = userDto.getMobilephone();

			if (newuser) {
				// 创建用户
				userService.saveToLdap(userDto, map, password);
			} else {
				// 更新用户
				userService.updateToLdap(userDto, map);
			}
			return result;
		} catch (BasException e) {
			throw new BasException(e.getMessage());
		}
	}

	@Override
	public BaseResult<UserDto> delete(String id) throws BasException {
		LdapPersonDto account = new LdapPersonDto();
		try {
			account.setAccount(id);
			userDao.insertUser(id);
			ldapPersonService.delete(account);
			userDao.deleteByPrimaryKey(id);
			return ResultUtils.build();
		} catch (LdapException e) {
			log.error(e.getMessage(), e);
			throw new UserException(e);
		}
	}

	@Override
	public BaseResult<PageCriteria<UserDto>> getTenantUser(QueryDTO query) throws BasException {
		try {
			String currentUserId = UserUtils.getCurrentUser();
			Example example = buildQuery(query, User.class);
			PageCriteria<UserDto>	result = query(query, () -> userDao.selectUserByEmpuid(example, currentUserId),
					UserDto.class,(Object dest, Object src) -> this.buildDTO(dest, src));
			return ResultUtils.build(result);
		} catch (BasException e) {
			log.error(e.getMessage(), e);
			throw new UserException(e);
		}
	}
	
	private void buildDTO(Object dest, Object src) {
		try {
			UserDto userdto = (UserDto) dest;
			super.commonBuild(dest, src);

			OrganizationDto org = orgService.getUserDept(userdto.getEmpuid());
			if (org != null) {
				userdto.setDeptid(org.getId());
				userdto.setDeptname(org.getName());
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}


	@Override
	public BaseResult<PageCriteria<UserDto>> query(QueryDTO query) throws BasException {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BaseResult<UserDto> get(String id) throws BasException {
		// TODO Auto-generated method stub
		return null;
	}
	@Override
	public boolean importemployee(MultipartFile file) throws BasException {
		String currentUserId = UserUtils.getCurrentUser();
		BaseResult<UserDto> BaseCurrentUserDto = userService.get(currentUserId);
		if (BaseCurrentUserDto == null || BaseCurrentUserDto.getData() == null) {
			throw new BasException("请先登陆！");
		}
		UserDto currentUserDto = BaseCurrentUserDto.getData();
		String tenantid = currentUserDto.getTenantid();
		if (tenantid == null) {
			tenantid = "0000000000";
		}

		InputStream in = null ;
		try {
			in = file.getInputStream();
		} catch (IOException e1) {
			throw new BasException("导入发送异常，请重试!");
		}
		List<UserDto> userList = new ArrayList<UserDto>();
		Workbook workBook = null;
		try {
			workBook = WorkbookFactory.create(in);
		} catch (Exception e) {
			throw new BasException("导入发送异常，请重试!");
		}
		Sheet sheet = workBook.getSheetAt(0);
		int numOfRows = sheet.getLastRowNum() + 1; // 总行数
		SimpleDateFormat sDateFormat=new SimpleDateFormat("yyyy-MM-dd");
		List<String> mobilephoneEmailList = new ArrayList<String>();
		Map<String, String> mobilephoneEmailMap = new HashMap<String, String>();
		// 解析excel
		for (int i = 1; i < numOfRows; i++) {
			// i=1,表示从表格中第2行开始读取
			Row row = sheet.getRow(i);
			if (null == row) {
				continue;
			}
			UserDto userDto = new UserDto();

			Cell cell = row.getCell((short) 1); // 姓名
			String cnname = getImportCellValue(cell);
			cell = row.getCell((short) 2);  // 手机号
			String mobilephone = getImportCellValue(cell);
			cell = row.getCell((short) 3); // 邮箱
			String email = getImportCellValue(cell);
			cell = row.getCell((short) 4); // 入职日期
			String entrytimestr  = getImportCellValue(cell);
			Date entrytime = null;
			cell = row.getCell((short) 5); // 年龄
			String age = getImportCellValue(cell);
			cell = row.getCell((short) 6); // 直属领导
			String superior = getImportCellValue(cell);
			cell = row.getCell((short) 7); // 内部账号
			String empid = getImportCellValue(cell);
			cell = row.getCell((short) 8); // 性别
			String sex = getImportCellValue(cell);
			cell = row.getCell((short) 9); // 状态
			String isDeleted = getImportCellValue(cell);
			if(StringUtils.isEmpty(cnname)){
				throw new BasException("第"+(i+1)+"行姓名不能为空!");
			}
			if(StringUtils.isEmpty(mobilephone)){
				throw new BasException("第"+(i+1)+"行手机号不能为空!");
			}else{
				if(mobilephone.trim().length()<11){
					throw new BasException("第"+(i+1)+"行手机号长度不能小于11位!");
				}else if(mobilephoneEmailList.contains(mobilephone)){
					throw new BasException("第"+(i+1)+"行与第"+mobilephoneEmailMap.get(mobilephone)+"行手机号重复!");
				}
				try {
					Long.parseLong(mobilephone);
				} catch (Exception e) {
					throw new BasException("第"+(i+1)+"行手机号格式不正确,必须由数字组成!");
				}
				List<Map<String, Object>> employeeList = userDao.getEmployeesByMobilephoneEmpuidEmail(mobilephone);
				if(employeeList.size()>0){
					throw new BasException("第"+(i+1)+"行手机号已经存在!");
				}
			}
			if(StringUtils.isEmpty(email)){
				throw new BasException("第"+(i+1)+"行邮箱不能为空!");
			}else{
				if(!email.trim().contains("@")){
					throw new BasException("第"+(i+1)+"行邮格式不正确!");
				}
				if(mobilephoneEmailList.contains(email)){
					throw new BasException("第"+(i+1)+"行与第"+mobilephoneEmailMap.get(email)+"行邮格重复!");
				}
				List<Map<String, Object>> employeeList = userDao.getEmployeesByMobilephoneEmpuidEmail(mobilephone);
				if(employeeList.size()>0){
					throw new BasException("第"+(i+1)+"行邮格已经存在!");
				}
			}
			if(StringUtils.isEmpty(entrytimestr)){
				throw new BasException("第"+(i+1)+"行入职日期不能为空!");
			}else{
				try {
					entrytime=sDateFormat.parse(entrytimestr);
				} catch (Exception e) {
					throw new BasException("第"+(i+1)+"行入职日期不能为空!");
				}
			}
			if(StringUtils.isEmpty(superior)){
				throw new BasException("第"+(i+1)+"行直属领导不能为空!");
			}else if(superior.equals(mobilephone)){
				throw new BasException("第"+(i+1)+"行直属领导不能是本人!");
			}
			if(StringUtils.isEmpty(sex)){
				throw new BasException("第"+(i+1)+"行性别不能为空!");
			}else if(!"男".equals(sex)&&!"女".equals(sex)){
				throw new BasException("第"+(i+1)+"行性别输入错误,请选择男或者女");
			}
			if(StringUtils.isEmpty(isDeleted)){
				throw new BasException("第"+(i+1)+"行状态不能为空!");
			}else if("启用".equals(isDeleted)){
				isDeleted = "N";
			}else if("禁用".equals(isDeleted)){
				isDeleted = "Y";
			}else{
				throw new BasException("第"+(i+1)+"行状态错误,请选择启用或者禁用");
			}
			mobilephoneEmailList.add(mobilephone);
			mobilephoneEmailList.add(email);
			mobilephoneEmailMap.put(mobilephone, (i+1)+"");
			mobilephoneEmailMap.put(email, (i+1)+"");
			userDto.setCnname(cnname);
			userDto.setDisplayname(cnname);
			userDto.setMobilephone(mobilephone);
			userDto.setEmail(email);
//			userDto.setAge(age);
			userDto.setSuperior(superior);
			userDto.setEmpid(empid);
			userDto.setSex(sex);
			userDto.setIs_Deleted(isDeleted);
			userDto.setIsDeleted(isDeleted);
			userDto.setEntrytime(entrytime);
			userList.add(userDto);
		}
		for(int i = 0 ; i < userList.size() ; i++ ){
			UserDto userDto =  userList.get(i);
			String superior =userDto.getSuperior();
			List<Map<String, Object>> employeeList = userDao.getEmployeesByMobilephoneEmpuidEmail(superior);
			if(employeeList.size()==0&&!mobilephoneEmailList.contains(superior)){
				throw new BasException("第"+(i+2)+"行直属领导不存在!");
			}
		}
		for(int i = 0 ; i < userList.size() ; i++ ){
			UserDto userDto =  userList.get(i);
			String mobilephone =userDto.getMobilephone();
			TenantEmployeeDto tenantEmployeeDto = new TenantEmployeeDto();
			tenantEmployeeDto.setTenantid(tenantid);
			tenantEmployeeDto.setMobilephone(mobilephone);
			tenantEmployeeService.save(tenantEmployeeDto);
			tenantEmployeeDto = tenantEmployeeService.getByMobilephone(mobilephone);
			String empuid = tenantEmployeeDto.getId();
			userDto.setEmpuid(empuid);
			userDto.setPassword("");
			userDto.setBusinesstype("01");
			userDto.setTenantid(tenantid);
			// 数据库保存用户
			userService.save(userDto);
		}

		// 同步到ldap
		for(int i = 0 ; i < userList.size() ; i++ ){
			UserDto userDto =  userList.get(i);
			String mobilephone =userDto.getMobilephone();
			Map<String, String> map = new HashMap<String, String>();
			map.put("gidNumber", "100");
			map.put("homeDirectory", "/home/admin");
			map.put("sn", userDto.getEmpuid());
			map.put("uid", userDto.getEmpuid());
			map.put("uidNumber", "200");
			map.put("mail", userDto.getEmail());
			map.put("mobile", userDto.getMobilephone());
			map.put("givenName", userDto.getCnname());
			map.put("displayName", userDto.getDisplayname());
			userService.saveToLdap(userDto, map, mobilephone);
		}
		// 更新上级领导
		for(int i = 0 ; i < userList.size() ; i++ ){
			UserDto userDto =  userList.get(i);
			String mobilephone =userDto.getSuperior();
			List<Map<String, Object>> employeeList = userDao.getEmployeesByMobilephoneEmpuidEmail(mobilephone);
			String superiorname = "";
			String superior = "";
			if(employeeList.size()>0){
				superiorname = employeeList.get(0).get("CNNAME").toString();
				superior = employeeList.get(0).get("EMPUID").toString();
			}
			userDao.updateSuperior(userDto.getEmpuid(), superiorname, superior);
		}
		// 排序保存的顺序，优先保存需要
		return false;
	}
	
	private String getImportCellValue(Cell cell) {
		cell.setCellType(1);
		if(cell != null) {
			cell.setCellType(HSSFCell.CELL_TYPE_STRING);
			return cell.getStringCellValue();
		}
		return null;
	}

}
