package com.ekingwin.bas.cloud.user.web;

import com.ekingwin.bas.cloud.infra.dto.BaseResult;
import com.ekingwin.bas.cloud.infra.dto.QueryDTO;
import com.ekingwin.bas.cloud.infra.dto.SaveResultDTO;
import com.ekingwin.bas.cloud.infra.utils.db.PageCriteria;
import com.ekingwin.bas.cloud.infra.utils.web.UserUtils;
import com.ekingwin.bas.cloud.infra.utils.web.WebUtils;
import com.ekingwin.bas.cloud.org.dto.OrganizationDto;
import com.ekingwin.bas.cloud.user.UserException;
import com.ekingwin.bas.cloud.user.dto.*;
import com.ekingwin.bas.cloud.user.service.IUserMetaService;
import com.ekingwin.bas.cloud.user.service.IUserRoleOrgService;
import com.ekingwin.bas.cloud.util.ExportExcelUtils;
import com.ekingwin.bas.cloud.util.Result;
import com.github.pagehelper.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * Created by liuch on 2017-5-22.
 */
@RestController
@RequestMapping("/user")
@Api("用户userController相关增删改查的基本方法api")
public class UserController {

	@Autowired
	@Qualifier(value = "userMetaServiceImpl")
	private IUserMetaService userService;
	@Autowired
	private IUserRoleOrgService userRoleOrgService;



//	@RequestMapping(value = "/query", method = RequestMethod.POST)
//	@ApiOperation("根据过滤条件查询用户")
//	public BaseResult<PageCriteria<UserDto>> queryUser(@RequestBody(required = false) QueryDTO query) throws Exception {
//		return userService.query(query);
//	}

	@RequestMapping(value = "/query", method = RequestMethod.POST)
	@ApiOperation(value = "查询分页",response = InnerUserDto.class)
	public Result selectPage(@RequestBody InnerUserPageDto innerUserPageDto) throws Exception {
		PageInfo<InnerUserDto> pageInfo  = userService.selectPage(innerUserPageDto);
		return Result.ok(pageInfo);
	}


	@RequestMapping(value = "/deltete/{userId:.+}", method = RequestMethod.POST)
	public BaseResult<UserDto> deleteUser(@PathVariable(value = "userId")String userId) throws Exception {
		return userService.delete(userId);
	}

	@RequestMapping(value = "/batchForbidden", method = RequestMethod.POST)
	@ApiOperation("批量启用用户/暂停用户")
	public BaseResult<UserDto> batchForbidden(@RequestBody UserDto user) throws Exception {
		return userService.batchForbidden(user);

	}

	@RequestMapping(value = "/save", method = RequestMethod.POST)
	@ApiOperation("添加用户/修改用户")
	public BaseResult<UserDto> saveUser(@RequestBody UserDto user) throws Exception {
		return	userService.save(user);

	}



	@RequestMapping(value = "/get/{userId:.+}", method = RequestMethod.GET)
	public BaseResult<UserDto> getUser(@PathVariable(value = "userId")String userId) throws Exception {
		return userService.get(userId);
	}

	@RequestMapping(value = "/getCurrentUser", method = RequestMethod.GET)
	public String getCurrentUser() throws Exception {
		return UserUtils.getCurrentUser();
	}

	/**
	 * 获取当前登陆人信息
	 * @return
	 * @throws Exception
	 */
	@RequestMapping(value = "/getCurrentUserInfo", method = RequestMethod.GET)
	public BaseResult<UserDto> getCurrentUserInfo() throws Exception {
		String userId = UserUtils.getCurrentUser();
		return userService.get(userId);
	}





	@RequestMapping(value = "/exportsuser", method = RequestMethod.POST)
	@ApiOperation("导出用户")
	public ModelAndView exportsUser(@RequestBody(required = false) QueryDTO query,HttpServletRequest request,HttpServletResponse response) throws Exception {
		try {
            UserDto userDto = new UserDto();
            // 查出用户数据
            String logid = query.getAttr().get("logid");
    		int flag = Integer.parseInt(query.getAttr().get("flag"));
    		if (StringUtils.isEmpty(logid)) {
    			logid=UserUtils.getCurrentUser();
    		}
            PageCriteria<UserDto> plist = userService.queryList(query,logid,flag);
            List<UserDto> userlist = plist.getList();
            String title ="用户信息表";
            String[] rowsName=new String[]{"员工姓名","员工账号","员工手机号码","员工邮箱","员工部门","员工主岗位","员工兼岗","员工状态","员工类型"};
            List<Object[]>  dataList = new ArrayList<Object[]>();
            Object[] objs = null;
            for (int i = 0; i < userlist.size(); i++) {
                UserDto po =userlist.get(i);
                objs = new Object[rowsName.length];
                objs[0] = po.getCnname();
                objs[1] = po.getEmpid();
                objs[2] = po.getMobilephone();
                objs[3] = po.getEmail();
                objs[4] = po.getDeptname();
                objs[5] = po.getDefaultposname();
                objs[6] = userService.getAndPostByUserid(po.getEmpuid());
                String status="禁用";
                if (po.getStatus().equals("1") && po.getLockstatus().equals("1")) {
					status="启用";
				}
                objs[7] = status;
                String userType="";
                if (!StringUtils.isEmpty(userDto.getType())) {
                	if (userDto.getType().equals("EMP")) {
                		userType="内部用户";
                	}else if (userDto.getType().equals("ExtUser")) {
                		userType="外部用户";
                	}else if (userDto.getType().equals("SYSUser")) {
                		userType="系统账号";
                	}else if (userDto.getType().equals("Temp")) {
                		userType="临时用户";
                	}
				}
                objs[8]=userType;
                dataList.add(objs);
            }
            //
            ExportExcelUtils ex =new ExportExcelUtils(title, rowsName, dataList,response);
            ex.exportData();


        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
//		return userService.exportsUser(query);
	}

	@RequestMapping(value = "/importsuser", method = RequestMethod.POST)
	@ApiOperation("导入用户")
	public SaveResultDTO importsUser(@RequestParam(value="file") MultipartFile file) throws Exception {
		SaveResultDTO saveResultDTO = new SaveResultDTO();
		try {
			 if (file == null) {
				 saveResultDTO.setMessage("文件为空");
				 saveResultDTO.setResult("0");
				 return saveResultDTO;
			 }
	        String name = file.getOriginalFilename();// 获取上传文件名,包括路径
	        long size = file.getSize();
	        if (StringUtils.isEmpty(name) && size == 0){
	        	saveResultDTO.setMessage("文件为空");
				saveResultDTO.setResult("0");
				return saveResultDTO;
	        }
	        InputStream in = file.getInputStream();
	        userService.importsUser(in);
	        saveResultDTO.setResult("0");
	        saveResultDTO.setMessage("导入成功");
		} catch (Exception e) {
			// TODO: handle exception
		}
		return saveResultDTO;
	}

	@RequestMapping(value = "/queryUserRoleOrg", method = RequestMethod.POST)
	@ApiOperation("查询用户的角色岗位等")
	public PageCriteria<ViewUserRoleOrgDTO> queryUserRoleOrg(@RequestBody(required = false) QueryDTO query) throws Exception{
		return userRoleOrgService.queryUserRoleOrg(query);
	}

	@RequestMapping(value = "/queryUserNotBen", method = RequestMethod.POST)
	@ApiOperation("查询当前用户没祝福的员工列表")
	public PageCriteria<UserDto> queryUserByNotBen(@RequestBody(required = false) QueryDTO query) throws Exception {
		return userService.getUsersByNotBen(query);
	}

	@RequestMapping(value = "/queryParents", method = RequestMethod.POST)
	@ApiOperation("获取用户的上一级上司")
	public PageCriteria<UserDto> getParentUserById(@RequestBody(required = false) QueryDTO query) throws Exception {
		return userService.getParentUserById(query);
	}

	@RequestMapping(value = "/queryUserByUser", method = RequestMethod.POST)
	@ApiOperation("获取当前用户下的所有用户")
	public PageCriteria<UserDto> queryUserByUser(@RequestBody(required = false) QueryDTO query) throws Exception {
		return userService.getUsersByUser(query);
	}




	@RequestMapping(value = "/getUserByOrgidAndLogid", method = RequestMethod.POST)
	@ApiOperation("根据登录id或组织id来查询在组织或岗位下的用户")
	public PageCriteria<UserPostDto> getUserByOrgidAndLogid(@RequestBody(required = false) QueryDTO query) throws UserException{
		String orgid = query.getAttr().get("orgid");
		String logid = query.getAttr().get("logid");
		String flag = query.getAttr().get("flag");
		int num = 0;
		if (!StringUtils.isEmpty(flag)) {
			num = Integer.parseInt(flag);
		}
		if (StringUtils.isEmpty(logid)) {
			logid = UserUtils.getCurrentUser();
		}
		return userRoleOrgService.getUserByOrgidAndLogid(query,orgid,logid,num);
	}

	@RequestMapping(value = "/judgeAdminByLogid/{logid}", method = RequestMethod.GET)
	@ApiOperation("根据登录id判断是否是IDM超级管理员")
	public int getUserByOrgidAndLogid(@PathVariable(value = "logid") String logid) throws UserException{
		if (StringUtils.isEmpty(logid)) {
			logid = UserUtils.getCurrentUser();
		}
		return userService.judgeAdminByUserId(logid);
	}

	@RequestMapping(value = "/getAccountMappingByAccAndSyscode", method = RequestMethod.POST)
	@ApiOperation("根据输入账号和系统编码查询是否有该账户未映射信息")
	public AccountDispose getAccountMappingByAccAndSyscode(@RequestBody Map<String, String> map){
		try {
			String account = map.get("account");
			String syscode = map.get("syscode");
			if (!StringUtils.isEmpty(account) && !StringUtils.isEmpty(syscode)) {
				return userService.getAccountMappingByAccAndSyscode(account,syscode);
			}else{
				return null;
			}
		} catch (Exception e) {
			e.getMessage();
		}
		return null;
	}

	@RequestMapping(value = "/deleteAccountByAccAndSyscode", method = RequestMethod.DELETE)
	@ApiOperation("根据输入账号和系统编码删除该账户未映射信息")
	public boolean deleteAccountByAccAndSyscode(@RequestBody Map<String, String> map){
		try {
			String account = map.get("account");
			String syscode = map.get("syscode");
			if (!StringUtils.isEmpty(account) && !StringUtils.isEmpty(syscode)) {
				return userService.deleteAccountByAccAndSyscode(account,syscode);
			}else{
				return false;
			}
		} catch (Exception e) {
			e.getMessage();
		}
		return false;
	}




	@RequestMapping(value = "/exportsOrg", method = RequestMethod.POST)
	@ApiOperation("导出组织")
	public ModelAndView exportsOrg(@RequestBody(required = false) QueryDTO query,HttpServletRequest request,HttpServletResponse response) throws Exception {
		try {
			String orgid = query.getAttr().get("orgid");
			String logid = query.getAttr().get("logid");
			String flag = query.getAttr().get("flag");
			int num = 0;
			if (!StringUtils.isEmpty(flag)) {
				num = Integer.parseInt(flag);
			}
			if (StringUtils.isEmpty(logid)) {
				logid = UserUtils.getCurrentUser();
			}
			PageCriteria<UserPostDto> plist = userRoleOrgService.getUserByOrgidAndLogid(query,orgid,logid,num);
            List<UserPostDto> userlist = plist.getList();
            String title ="组织信息表";
            String[] rowsName=new String[]{"员工账号","员工姓名","员工手机号码","邮箱","组织","岗位","岗位类型"};
            List<Object[]>  dataList = new ArrayList<Object[]>();
            Object[] objs = null;
            for (int i = 0; i < userlist.size(); i++) {
            	UserPostDto po =userlist.get(i);
                objs = new Object[rowsName.length];
                objs[0] = po.getEmpuid();
                objs[1] = po.getCnname();
                objs[2] = po.getMobilephone();
                objs[3] = po.getEmail();
                objs[4] = po.getDeptname();
                objs[5] = po.getDefaultposname();
                String postType="主岗";
                if (!StringUtils.isEmpty(po.getIsmainpost())) {
					if (po.getIsmainpost().equals("S")) {
						postType="兼岗";
					}
				}else{
					postType="--";
				}
                objs[6] = postType;
                dataList.add(objs);
            }
            //
            ExportExcelUtils ex =new ExportExcelUtils(title, rowsName, dataList,response);
            ex.exportData();


        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
//		return userService.exportsUser(query);
	}

	@RequestMapping(value = "/getUserInfo/{userId}", method = RequestMethod.GET)
	@ApiOperation("根据用户id获取用户信息（包含主岗岗位全路径）")
	public UserDto getUserInfo(@PathVariable(value = "userId")String userId) throws Exception {
		try {
			if (!StringUtils.isEmpty(userId)) {
				return userService.getUserInfo(userId);
			}
		} catch (Exception e) {
			e.getMessage();
		}
		return null;
	}




	@RequestMapping(value = "/exportsPostInfo", method = RequestMethod.POST)
	@ApiOperation("导出岗位信息参照表")
	public ModelAndView exportsOrg(HttpServletRequest request,HttpServletResponse response) throws Exception {
		try {

            List<OrganizationDto> userlist = userService.getPostInfo();
            String title ="岗位信息参照表";
            String[] rowsName=new String[]{"岗位id","岗位全路径"};
            List<Object[]>  dataList = new ArrayList<Object[]>();
            Object[] objs = null;
            for (int i = 0; i < userlist.size(); i++) {
            	OrganizationDto po =userlist.get(i);
                objs = new Object[rowsName.length];
                objs[0] = po.getId();
                dataList.add(objs);
            }
            //
            ExportExcelUtils ex =new ExportExcelUtils(title, rowsName, dataList,response);
            ex.exportData();


        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
	}


	@RequestMapping(value = "/exportsUserReference", method = RequestMethod.POST)
	@ApiOperation("导出用户导入参照表")
	public ModelAndView exportsUserReference(HttpServletRequest request,HttpServletResponse response) throws Exception {
		try {
            String title ="导入员工模板表";
            String[] rowsName=new String[]{"员工姓名","员工电话","员工邮箱","员工岗位id","员工类型"};
            List<Object[]>  dataList = new ArrayList<Object[]>();
            Object[] objs = null;
            for (int i = 0; i < 1; i++) {
                objs = new Object[rowsName.length];
                objs[0] = "张三";
                objs[1] = "18788889888";
                objs[2] = "zhangsan@cq-p.com.cn";
                objs[3] = "2921";
                objs[4] = "内部";
                dataList.add(objs);
            }
            //
            ExportExcelUtils ex =new ExportExcelUtils(title, rowsName, dataList,response);
            ex.exportData();


        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
	}

	@RequestMapping(value = "/syncuser", method = RequestMethod.POST)
	public SaveResultDTO syncUser(@RequestBody List<UserDto> listUserDto) throws Exception {
		SaveResultDTO result = new SaveResultDTO();
		try {
		 for(UserDto userDto:listUserDto){
			userService.save(userDto);
			result.setResult("1");
			result.setMessage(userDto.getEmpuid());
		 }
		} catch (Exception e) {
			result.setResult("0");
			result.setMessage(e.getMessage());
		}
		return result;

	}

	@RequestMapping(value = "/modifyPasswordByOldPassword", method = RequestMethod.POST)
	public BaseResult<PasswordInfoDto> modifyPasswordByOldPassword(@RequestBody PasswordInfoDto passwordInfoDto) throws Exception {
		BaseResult<PasswordInfoDto> baseResult = new BaseResult<PasswordInfoDto>();
		baseResult.setTimestamp(String.valueOf(new Date().getTime()));
		baseResult.setPath(WebUtils.getRequest().getRequestURI());
		try {
			userService.modifyPasswordByOldPassword(passwordInfoDto);
			baseResult.setStatus("200");
			baseResult.setData(passwordInfoDto);
		} catch (Exception e) {
			baseResult.setStatus("500");
			baseResult.setMessage(e.getMessage());
		}
		return baseResult;

	}

	@RequestMapping(value = "/modifyPasswordByVerificationCode", method = RequestMethod.POST)
	public BaseResult<PasswordInfoDto> modifyPasswordByVerificationCode(@RequestBody PasswordInfoDto passwordInfoDto) throws Exception {
		BaseResult<PasswordInfoDto> baseResult = new BaseResult<PasswordInfoDto>();
		baseResult.setTimestamp(String.valueOf(new Date().getTime()));
		baseResult.setPath(WebUtils.getRequest().getRequestURI());
		try {
			userService.modifyPasswordByVerificationCode(passwordInfoDto);
			baseResult.setStatus("200");
			baseResult.setData(passwordInfoDto);
		} catch (Exception e) {
			baseResult.setStatus("500");
			baseResult.setMessage(e.getMessage());
		}
		return baseResult;
	}

	@RequestMapping(value = "/loginVerification", method = RequestMethod.POST)
	public BaseResult<LoginVerificationDto> loginVerification(@RequestBody LoginVerificationDto loginVerificationDto) throws Exception {
		BaseResult<LoginVerificationDto> baseResult = new BaseResult<LoginVerificationDto>();
		baseResult.setTimestamp(String.valueOf(new Date().getTime()));
		baseResult.setPath(WebUtils.getRequest().getRequestURI());
		try {
			userService.loginVerification(loginVerificationDto);
			baseResult.setStatus("200");
			baseResult.setData(loginVerificationDto);
		} catch (Exception e) {
			baseResult.setStatus("500");
			baseResult.setMessage(e.getMessage());
		}
		return baseResult;
	}
	@RequestMapping(value = "/caslogin", method = RequestMethod.GET)
	public String caslogin() throws Exception {
		return "";
	}
	@RequestMapping(value = "/login", method = RequestMethod.GET)
	public String login() throws Exception {
		return "";
	}

}
